@@ -269,7 +269,7 @@ void joy_configure(void)
 		gtk_button_box_set_spacing(GTK_BUTTON_BOX(box), 5);
 		gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 0);
 
-		button = gtk_button_new_with_label(_("Ok"));
+		button = gtk_button_new_with_label(_("OK"));
 		gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(joyconf_ok_cb), NULL);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
