@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2003  Haavard Kvaalen <havardk@xmms.org>
+ *  Copyright (C) 1999-2006  Haavard Kvaalen <havardk@xmms.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -310,7 +310,7 @@ static gboolean is_mounted(char * device_name)
 #endif
 
 	if (lstat(device_name, &st) < 0)
-		return -1;
+		return FALSE;
 
 	if (S_ISLNK(st.st_mode))
 		readlink(device_name, devname, 256);
@@ -333,7 +333,7 @@ static gboolean is_mounted(char * device_name)
 #elif defined(HAVE_GETMNTINFO)
  	entries = getmntinfo(&fsp, MNT_NOWAIT);
  	if (entries < 0)
- 		return NULL;
+ 		return FALSE;
 	
 	while (entries-- > 0)
 	{
@@ -488,6 +488,8 @@ static void play_file(char *filename)
 
 	if (!sscanf(tmp, "Track %d.cda", &track))
 		return;
+	if (track < 0 || track > 99)
+		return;
 
 	if (!cdda_get_toc(&cdda_playing.cd_toc, drive->device) ||
 	    cdda_playing.cd_toc.track[track].flags.data_track ||
