@@ -24,6 +24,7 @@
 #include <string.h>
 #include <errno.h>
 #include <libxmms/xentry.h>
+#include <gdk/gdkkeysyms.h>
 #include "mpg123.h"
 
 static GtkWidget *window = NULL;
@@ -52,11 +53,7 @@ G_GNUC_PRINTF(2, 3);
 
 static void set_entry_tag(GtkEntry * entry, char * tag, int length)
 {
-	int stripped_len;
-	char *text;
-
-	stripped_len = mpg123_strip_spaces(tag, length);
-	text = g_strdup_printf("%-*.*s", stripped_len, stripped_len, tag);
+	char *text = g_strchomp(g_strndup(tag, length));
 	gtk_entry_set_text(entry, text);
 	g_free(text);
 }
@@ -138,7 +135,7 @@ static void save_cb(GtkWidget * w, gpointer data)
 	if (msg)
 	{
 		GtkWidget *mwin = xmms_show_message(_("File Info"), msg,
-						    _("Ok"), FALSE, NULL, NULL);
+						    _("OK"), FALSE, NULL, NULL);
 		gtk_window_set_transient_for(GTK_WINDOW(mwin),
 					     GTK_WINDOW(window));
 		g_free(msg);
@@ -193,7 +190,7 @@ static void remove_id3_cb(GtkWidget * w, gpointer data)
 	if (msg)
 	{
 		GtkWidget *mwin = xmms_show_message(_("File Info"), msg,
-						    _("Ok"), FALSE, NULL, NULL);
+						    _("OK"), FALSE, NULL, NULL);
 		gtk_window_set_transient_for(GTK_WINDOW(mwin),
 					     GTK_WINDOW(window));
 		g_free(msg);
@@ -263,6 +260,12 @@ static void genre_set_popdown(GtkWidget *combo, GList *genres)
 	}
 }
 
+static void file_info_box_keypress_cb(GtkWidget *w, GdkEventKey *event, gpointer data)
+{
+	if (event && event->keyval == GDK_Escape)
+		gtk_widget_destroy(w);
+}
+
 void mpg123_file_info_box(char *filename)
 {
 	int i;
@@ -275,7 +278,7 @@ void mpg123_file_info_box(char *filename)
 	emphasis[0] = _("None");
 	emphasis[1] = _("50/15 ms");
 	emphasis[2] = "";
-	emphasis[3] = _("CCIT J.17");
+	emphasis[3] = _("CCITT J.17");
 	bool_label[0] = _("No");
 	bool_label[1] = _("Yes");
 
@@ -290,6 +293,8 @@ void mpg123_file_info_box(char *filename)
 		gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
 		gtk_signal_connect(GTK_OBJECT(window), "destroy",
 				   gtk_widget_destroyed, &window);
+		gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
+				   file_info_box_keypress_cb, NULL);
 		gtk_container_set_border_width(GTK_CONTAINER(window), 10);
 
 		vbox = gtk_vbox_new(FALSE, 10);
@@ -521,10 +526,13 @@ void mpg123_file_info_box(char *filename)
 
 	if ((fh = fopen(current_filename, "rb")) != NULL)
 	{
-		guint32 head;
-		unsigned char tmp[4];
 		struct frame frm;
 		gboolean id3_found = FALSE;
+		guint8 *buf;
+		double tpf;
+		int pos;
+		xing_header_t xing_header;
+		guint32 num_frames;
 
 		fseek(fh, -sizeof (tag), SEEK_END);
 		if (fread(&tag, 1, sizeof (tag), fh) == sizeof (tag))
@@ -541,7 +549,7 @@ void mpg123_file_info_box(char *filename)
 				set_entry_tag(GTK_ENTRY(year_entry),
 					      tag.year, 4);
 				/* Check for v1.1 tags */
-				if (tag.u.v1_1.__zero == 0)
+				if (tag.u.v1_1.__zero == 0 && tag.u.v1_1.track_number > 0)
 				{
 					char *temp = g_strdup_printf("%d", tag.u.v1_1.track_number);
 					set_entry_tag(GTK_ENTRY(comment_entry),
@@ -560,53 +568,46 @@ void mpg123_file_info_box(char *filename)
 			}
 		}
 		rewind(fh);
-		if (fread(tmp, 1, 4, fh) != 4)
+		
+		if (!mpg123_get_first_frame(fh, &frm, &buf))
 		{
 			fclose(fh);
 			return;
 		}
-		head = ((guint32) tmp[0] << 24) | ((guint32) tmp[1] << 16) | ((guint32) tmp[2] << 8) | (guint32) tmp[3];
-		while (!mpg123_head_check(head))
-		{
-			head <<= 8;
-			if (fread(tmp, 1, 1, fh) != 1)
-			{
-				fclose(fh);
-				return;
-			}
-			head |= tmp[0];
-		}
-		if (mpg123_decode_header(&frm, head))
-		{
-			guint8 *buf;
-			double tpf;
-			int pos;
-			xing_header_t xing_header;
-			guint32 num_frames;
 
-			buf = g_malloc(frm.framesize + 4);
-			fseek(fh, -4, SEEK_CUR);
-			fread(buf, 1, frm.framesize + 4, fh);
-			tpf = mpg123_compute_tpf(&frm);
-			set_mpeg_level_label(frm.mpeg25, frm.lsf, frm.lay);
-			pos = ftell(fh);
-			fseek(fh, 0, SEEK_END);
-			if (mpg123_get_xing_header(&xing_header, buf))
-			{
-				num_frames = xing_header.frames;
-				label_set_text(mpeg_bitrate, _("Bitrate: Variable,\navg. bitrate: %d kb/s"), (int)((xing_header.bytes * 8) / (tpf * xing_header.frames * 1000)));
-			}
-			else
-			{
-				num_frames = ((ftell(fh) - pos - (id3_found ? 128 : 0)) / mpg123_compute_bpf(&frm)) + 1;
-				label_set_text(mpeg_bitrate, _("Bitrate: %d kb/s"), tabsel_123[frm.lsf][frm.lay - 1][frm.bitrate_index]);
-			}
-			label_set_text(mpeg_samplerate, _("Samplerate: %d Hz"),
-				       mpg123_freqs[frm.sampling_frequency]);
-			label_set_text(mpeg_flags, _("%s\nError protection: %s\nCopyright: %s\nOriginal: %s\nEmphasis: %s"), channel_mode_name(frm.mode), bool_label[frm.error_protection], bool_label[frm.copyright], bool_label[frm.original], emphasis[frm.emphasis]);
-			label_set_text(mpeg_fileinfo, _("%d frames\nFilesize: %lu B"), num_frames, ftell(fh));
-			g_free(buf);
+		tpf = mpg123_compute_tpf(&frm);
+		set_mpeg_level_label(frm.mpeg25, frm.lsf, frm.lay);
+		pos = ftell(fh);
+		fseek(fh, 0, SEEK_END);
+		if (mpg123_get_xing_header(&xing_header, buf))
+		{
+			num_frames = xing_header.frames;
+			label_set_text(mpeg_bitrate,
+				       _("Bitrate: Variable,\n"
+					 "avg. bitrate: %d kb/s"),
+				       (int)((xing_header.bytes * 8) /
+					     (tpf * xing_header.frames * 1000)));
 		}
+		else
+		{
+			num_frames = ((ftell(fh) - pos - (id3_found ? 128 : 0)) /
+				      mpg123_compute_bpf(&frm)) + 1;
+			label_set_text(mpeg_bitrate,
+				       _("Bitrate: %d kb/s"),
+				       tabsel_123[frm.lsf][frm.lay - 1][frm.bitrate_index]);
+		}
+		label_set_text(mpeg_samplerate, _("Samplerate: %d Hz"),
+			       mpg123_freqs[frm.sampling_frequency]);
+		label_set_text(mpeg_flags, _("%s\nError protection: %s\n"
+					     "Copyright: %s\nOriginal: %s\n"
+					     "Emphasis: %s"),
+			       channel_mode_name(frm.mode),
+			       bool_label[frm.error_protection],
+			       bool_label[frm.copyright], bool_label[frm.original],
+			       emphasis[frm.emphasis]);
+		label_set_text(mpeg_fileinfo, _("%d frames\nFilesize: %lu B"),
+			       num_frames, ftell(fh));
+		g_free(buf);
 		fclose(fh);
 	}
 }
