@@ -205,7 +205,7 @@ static void show_error_message(gchar *error)
 	if(!error_dialog)
 	{
 		GDK_THREADS_ENTER();
-		error_dialog = xmms_show_message(_("Error"), error, _("Ok"), FALSE,
+		error_dialog = xmms_show_message(_("Error"), error, _("OK"), FALSE,
 						 NULL, NULL);
 		gtk_signal_connect(GTK_OBJECT(error_dialog),
 				   "destroy",
@@ -328,7 +328,8 @@ static void *http_buffer_loop(void *arg)
 	gchar line[1024], *user, *pass, *host, *filename,
 	     *status, *url, *temp, *file;
 	gchar *chost;
-	gint cnt, written, error, err_len, port, cport;
+	gint cnt, written, error, port, cport;
+	socklen_t err_len;
 	gboolean redirect;
 	int udp_sock = 0;
 	fd_set set;
@@ -890,7 +891,7 @@ static int udp_check_for_data(int sock)
 		else if (strstr(lines[i], "x-audiocast-streammsg") != NULL)
 		{
 			/*  mpg123_ip.set_info(title, -1, mpg123_bitrate * 1000, mpg123_frequency, mpg123_stereo); */
-/*  			xmms_show_message(_("Message"), valptr, _("Ok"), */
+/*  			xmms_show_message(_("Message"), valptr, _("OK"), */
 /*  					  FALSE, NULL, NULL); */
 			g_message("Stream_message: %s", valptr);
 		}
