@@ -1,30 +1,23 @@
-/*********************************************************************
- * 
+/*
  *    Copyright (C) 1999-2000, 2001,  Espen Skoglund
- *    Department of Computer Science, University of Troms
- * 
- * Filename:      id3_frame.c
- * Description:   Code for handling ID3 frames.
- * Author:        Espen Skoglund <espensk@stud.cs.uit.no>
- * Created at:    Fri Feb  5 23:47:08 1999
- * 
- * $Id: id3_frame.c,v 1.19 2004/02/22 17:30:02 havard Exp $
- * 
+ *    Copyright (C) 1999-2004  Haavard Kvaalen
+ *
+ * $Id: id3_frame.c,v 1.21 2004/04/04 16:14:31 havard Exp $
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *                
- ********************************************************************/
+ */
 #include "config.h"
 
 #ifdef HAVE_LIBZ
@@ -194,7 +187,7 @@ static struct id3_framedesc22 framedesc22[] = {
 
 	{ID3_SLT, ID3_SYLT}, /* Synchronized lyric/text */
 	{ID3_STC, ID3_SYTC}, /* Synced tempo codes */
-	 
+ 
 	{ID3_TAL, ID3_TALB}, /* Album/Movie/Show title */
 	{ID3_TBP, ID3_TBPM}, /* BPM (Beats Per Minute) */
 	{ID3_TCM, ID3_TCOM}, /* Composer */
@@ -248,7 +241,7 @@ static struct id3_framedesc22 framedesc22[] = {
 	{ID3_WCP, ID3_WCOP}, /* Copyright/Legal information */
 	{ID3_WPB, ID3_WPUB}, /* Publishers official webpage */
 	{ID3_WXX, ID3_WXXX}, /* User defined URL link frame */
-};	
+};
 
 static struct id3_framedesc *find_frame_description(guint32 id)
 {
@@ -258,7 +251,7 @@ static struct id3_framedesc *find_frame_description(guint32 id)
 			return &framedesc[i];
 	return NULL;
 }
-	
+
 
 /*
  * Function id3_read_frame (id3)
@@ -271,11 +264,11 @@ int id3_read_frame(struct id3_tag *id3)
 {
 	struct id3_frame *frame;
 	guint32 id;
-	char *buf;
+	guint8 *buf;
 
 	if (id3->id3_version == 2)
 		return id3_read_frame_v22(id3);
-    
+
 	/*
 	 * Read frame header.
 	 */
@@ -335,7 +328,7 @@ int id3_read_frame(struct id3_tag *id3)
 	/*
 	 * Initialize frame.
 	 */
-			
+
 	/*
 	 * We allocate 2 extra bytes.  This simplifies retrieval of
 	 * text strings.
@@ -347,7 +340,7 @@ int id3_read_frame(struct id3_tag *id3)
 		g_free(frame);
 		return -1;
 	}
-			
+
 	/*
 	 * Insert frame into linked list.
 	 */
@@ -358,7 +351,7 @@ int id3_read_frame(struct id3_tag *id3)
 	 */
 	if (frame->fr_flags & ID3_FHFLAG_COMPRESS)
 		return 0;
-    
+
 	frame->fr_data = id3_frame_get_dataptr(frame);
 	frame->fr_size = id3_frame_get_size(frame);
 
@@ -401,96 +394,99 @@ struct id3_frame *id3_get_frame(struct id3_tag *id3, guint32 type, int num)
 static int decompress_frame(struct id3_frame *frame)
 {
 #ifdef HAVE_LIBZ
-    z_stream z;
-    int r;
+	z_stream z;
+	int r;
 
-    /*
-     * Fetch the size of the decompressed data.
-     */
-    frame->fr_size_z = g_ntohl( *((guint32 *) frame->fr_raw_data) );
-    if (frame->fr_size_z < 0 || frame->fr_size_z > 1000000)
-	return -1;
+	/*
+	 * Fetch the size of the decompressed data.
+	 */
+	frame->fr_size_z = g_ntohl(*((guint32 *) frame->fr_raw_data));
+	if (frame->fr_size_z < 0 || frame->fr_size_z > 1000000)
+		return -1;
 
-    /*
-     * Allocate memory to hold uncompressed frame.
-     */
-    frame->fr_data_z = g_malloc(frame->fr_size_z + 
-				(id3_frame_is_text(frame) ? 2 : 0));
+	/*
+	 * Allocate memory to hold uncompressed frame.
+	 */
+	frame->fr_data_z = g_malloc(frame->fr_size_z +
+				    (id3_frame_is_text(frame) ? 2 : 0));
 
-    /*
-     * Initialize zlib.
-     */
-    z.next_in   = id3_frame_get_dataptr(frame);
-    z.avail_in  = id3_frame_get_size(frame);
-    z.zalloc    = NULL;
-    z.zfree     = NULL;
-    z.opaque    = NULL;
+	/*
+	 * Initialize zlib.
+	 */
+	z.next_in = id3_frame_get_dataptr(frame);
+	z.avail_in = id3_frame_get_size(frame);
+	z.zalloc = NULL;
+	z.zfree = NULL;
+	z.opaque = NULL;
 
-    r = inflateInit( &z );
-    switch ( r ) {
-    case Z_OK:
-	break;
-    case Z_MEM_ERROR:
-	id3_error( frame->fr_owner, "zlib - no memory" );
-	goto Error_init;
-    case Z_VERSION_ERROR:
-	id3_error( frame->fr_owner, "zlib - invalid version" );
-	goto Error_init;
-    default:
-	id3_error( frame->fr_owner, "zlib - unknown error" );
-	goto Error_init;
-    }
+	r = inflateInit(&z);
+	switch (r)
+	{
+		case Z_OK:
+			break;
+		case Z_MEM_ERROR:
+			id3_error(frame->fr_owner, "zlib - no memory");
+			goto Error_init;
+		case Z_VERSION_ERROR:
+			id3_error(frame->fr_owner, "zlib - invalid version");
+			goto Error_init;
+		default:
+			id3_error(frame->fr_owner, "zlib - unknown error");
+			goto Error_init;
+	}
 
-    /*
-     * Decompress frame.
-     */
-    z.next_out = frame->fr_data_z;
-    z.avail_out = frame->fr_size_z;
-    r = inflate( &z, Z_SYNC_FLUSH );
-    switch ( r ) {
-    case Z_STREAM_END:
-	break;
-    case Z_OK:
-	if ( z.avail_in == 0 )
-	    /*
-	     * This should not be possible with a correct stream.
-	     * We will be nice however, and try to go on.
-	     */
-	    break;
-	id3_error( frame->fr_owner, "zlib - buffer exhausted" );
-	goto Error_inflate;
-    default:
-	id3_error( frame->fr_owner, "zlib - unknown error" );
-	goto Error_inflate;
-    }
+	/*
+	 * Decompress frame.
+	 */
+	z.next_out = frame->fr_data_z;
+	z.avail_out = frame->fr_size_z;
+	r = inflate(&z, Z_SYNC_FLUSH);
+	switch (r)
+	{
+		case Z_STREAM_END:
+			break;
+		case Z_OK:
+			if (z.avail_in == 0)
+				/*
+				 * This should not be possible with a
+				 * correct stream.  We will be nice
+				 * however, and try to go on.
+				 */
+				break;
+			id3_error(frame->fr_owner, "zlib - buffer exhausted");
+			goto Error_inflate;
+		default:
+			id3_error(frame->fr_owner, "zlib - unknown error");
+			goto Error_inflate;
+	}
 
-    r = inflateEnd( &z );
-    if ( r != Z_OK )
-	id3_error( frame->fr_owner, "zlib - inflateEnd error" );
+	r = inflateEnd(&z);
+	if (r != Z_OK)
+		id3_error(frame->fr_owner, "zlib - inflateEnd error");
 
-    /*
-     * Null-terminate text frames.
-     */
-    if (id3_frame_is_text(frame))
-    {
-	((char *) frame->fr_data_z)[frame->fr_size_z] = 0;
-	((char *) frame->fr_data_z)[frame->fr_size_z + 1] = 0;
-    }
-    frame->fr_data = frame->fr_data_z;
-    frame->fr_size = frame->fr_size_z + (id3_frame_is_text(frame) ? 2 : 0);
+	/*
+	 * Null-terminate text frames.
+	 */
+	if (id3_frame_is_text(frame))
+	{
+		((char *) frame->fr_data_z)[frame->fr_size_z] = 0;
+		((char *) frame->fr_data_z)[frame->fr_size_z + 1] = 0;
+	}
+	frame->fr_data = frame->fr_data_z;
+	frame->fr_size = frame->fr_size_z + (id3_frame_is_text(frame) ? 2 : 0);
 
-    return 0;
+	return 0;
 
-    /*
-     * Cleanup code.
-     */
+	/*
+	 * Cleanup code.
+	 */
  Error_inflate:
-    r = inflateEnd( &z );
+	r = inflateEnd(&z);
  Error_init:
-    g_free(frame->fr_data_z);
-    frame->fr_data_z = NULL;
+	g_free(frame->fr_data_z);
+	frame->fr_data_z = NULL;
 #endif
-    return -1;
+	return -1;
 }
 
 /*
@@ -521,38 +517,39 @@ int id3_decompress_frame(struct id3_frame *frame)
  */
 int id3_delete_frame(struct id3_frame *frame)
 {
-    GList *list = frame->fr_owner->id3_frame;
-    int ret;
+	GList *list = frame->fr_owner->id3_frame;
+	int ret;
 
-    /*
-     * Search for frame in list.
-     */
-
-    if (g_list_find(list, frame) != NULL) {
 	/*
-	 * Frame does not exist in frame list.
+	 * Search for frame in list.
 	 */
-	ret = -1;
+	if (g_list_find(list, frame) != NULL)
+	{
+		/*
+		 * Frame does not exist in frame list.
+		 */
+		ret = -1;
+	}
+	else
+	{
+		/*
+		 * Remove frame from frame list.
+		 */
+		list = g_list_remove(list, frame);
+		frame->fr_owner->id3_altered = 1;
+		ret = 0;
+	}
 
-    } else {
 	/*
-	 * Remove frame from frame list.
+	 * Release memory occupied by frame.
 	 */
-	list = g_list_remove(list, frame);
-	frame->fr_owner->id3_altered = 1;
-	ret = 0;
-    }
+	if (frame->fr_raw_data)
+		g_free(frame->fr_raw_data);
+	if (frame->fr_data_z)
+		g_free(frame->fr_data_z);
+	g_free(frame);
 
-    /*
-     * Release memory occupied by frame.
-     */
-    if (frame->fr_raw_data)
-	 g_free(frame->fr_raw_data);
-    if ( frame->fr_data_z )
-	 g_free( frame->fr_data_z );
-    g_free( frame );
-
-    return ret;
+	return ret;
 }
 
 
@@ -565,36 +562,38 @@ int id3_delete_frame(struct id3_frame *frame)
  */
 struct id3_frame *id3_add_frame(struct id3_tag *id3, guint32 type)
 {
-    struct id3_frame *frame;
-    int i;
+	struct id3_frame *frame;
+	int i;
 
-    /*
-     * Allocate frame.
-     */
-    frame = g_malloc0( sizeof(*frame) );
+	/*
+	 * Allocate frame.
+	 */
+	frame = g_malloc0(sizeof(*frame));
 
-    /*
-     * Initialize frame
-     */
-    frame->fr_owner = id3;
+	/*
+	 * Initialize frame
+	 */
+	frame->fr_owner = id3;
 
-    /*
-     * Try finding the correct frame descriptor.
-     */
-    for (i = 0; i < sizeof(framedesc) / sizeof(struct id3_framedesc); i++) {
-	if (framedesc[i].fd_id == type) {
-	    frame->fr_desc = &framedesc[i];
-	    break;
+	/*
+	 * Try finding the correct frame descriptor.
+	 */
+	for (i = 0; i < sizeof(framedesc) / sizeof(struct id3_framedesc); i++)
+	{
+		if (framedesc[i].fd_id == type)
+		{
+			frame->fr_desc = &framedesc[i];
+			break;
+		}
 	}
-    }
 
-    /*
-     * Insert frame into linked list.
-     */
-    id3->id3_frame = g_list_append(id3->id3_frame, frame);
-    id3->id3_altered = 1;
+	/*
+	 * Insert frame into linked list.
+	 */
+	id3->id3_frame = g_list_append(id3->id3_frame, frame);
+	id3->id3_altered = 1;
 
-    return frame;
+	return frame;
 }
 
 
@@ -636,7 +635,7 @@ static int id3_frame_extra_headers(struct id3_frame *frame)
 	 */
 	if (frame->fr_flags & ID3_FHFLAG_ENCRYPT)
 		retv += 1;
-	
+
 	/*
 	 * If frame has grouping identity, we have one extra byte in
 	 * the header.
@@ -652,7 +651,7 @@ static void* id3_frame_get_dataptr(struct id3_frame *frame)
 	char *ptr = frame->fr_raw_data;
 
 	ptr += id3_frame_extra_headers(frame);
-	
+
 	return ptr;
 }
 
@@ -753,7 +752,7 @@ static int id3_read_frame_v22(struct id3_tag *id3)
 	 * Insert frame into linked list.
 	 */
 	id3->id3_frame = g_list_append(id3->id3_frame, frame);
-	
+
 	frame->fr_data = frame->fr_raw_data;
 	frame->fr_size = frame->fr_raw_size;
 
