@@ -29,11 +29,13 @@
 #include <pthread.h>
 #include <glib.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <ogg/ogg.h>
 #include <vorbis/codec.h>
 #include <vorbis/vorbisfile.h>
 
 #include "libxmms/util.h"
+#include "libxmms/charset.h"
 #include <xmms/i18n.h>
 
 #include "vorbis.h"
@@ -65,6 +67,12 @@ static GtkWidget *rg_track_entry, *rg_album_entry, *rg_track_peak_entry, *rg_alb
 static GtkWidget *rg_track_label, *rg_album_label, *rg_track_peak_label, *rg_album_peak_label;
 static GtkWidget *rg_show_button;
 
+static GtkWidget *bitrate_label, *avgbitrate_label, *rate_label;
+static GtkWidget *channel_label, *length_label, *filesize_label;
+static GtkWidget *replaygain_label, *audiophilegain_label, *peak_label;
+static GtkWidget *filename_entry, *tag_frame, *vendor_label;
+
+
 /* From mpg123.c, as no standardized Ogg Vorbis genres exists. */
 static const gchar *vorbis_genres[] =
 {
@@ -117,7 +125,7 @@ static gchar* get_comment(vorbis_comment *vc, gchar *label)
 {
 	gchar *tag;
 	if (vc && (tag = vorbis_comment_query(vc, label, 0)) != NULL)
-		return convert_from_utf8(tag);
+		return xmms_charset_from_utf8(tag);
 	else
 		return g_strdup("");
 }
@@ -130,7 +138,6 @@ static char** get_comment_list(vorbis_comment *vc)
 	strv = g_new0(char*, vc->comments + 1);
 	for (i = 0; i < vc->comments; i++)
 	{
-		g_message(vc->user_comments[i]);
 		strv[i] = g_strdup(vc->user_comments[i]);
 	}
 
@@ -155,7 +162,7 @@ static char** add_tag(char **list, char *label, char *tag)
 			g_free(*ptr);
 			if (tag != NULL)
 			{
-				tag = convert_to_utf8(tag);
+				tag = xmms_charset_to_utf8(tag);
 				*ptr = g_strconcat(reallabel, tag, NULL);
 				g_free(tag);
 				tag = NULL;
@@ -178,7 +185,7 @@ static char** add_tag(char **list, char *label, char *tag)
 		for (ptr = list; *ptr; ptr++)
 			i++;
 		list = g_renew(char*, list, i + 2);
-		tag = convert_to_utf8(tag);
+		tag = xmms_charset_to_utf8(tag);
 		list[i] = g_strconcat(reallabel, tag, NULL);
 		list[i + 1] = NULL;
 		g_free(tag);
@@ -195,15 +202,12 @@ static void add_list(vorbis_comment *vc, char **comments)
 }
 
 
-static void fail(gchar *error)
+static void fail(char *error)
 {
-	gchar *errorstring;
-	errorstring = g_strdup_printf(_("An error occured:\n%s"), error);
+	char *errorstring;
+	errorstring = g_strdup_printf(_("An error occurred:\n%s"), error);
 	
-	xmms_show_message(
-		_("Error!"),
-		errorstring,
-		_("Ok"), FALSE, NULL, NULL);
+	xmms_show_message(_("Error!"), errorstring, _("OK"), FALSE, NULL, NULL);
 
 	g_free(errorstring);
 	return;
@@ -422,17 +426,24 @@ static void label_set_text(GtkWidget * label, char *str, ...)
 	g_free(tempstr);
 }
 
+static void keypress_cb(GtkWidget * w, GdkEventKey * event, gpointer data)
+{
+	if (event && event->keyval == GDK_Escape)
+		gtk_widget_destroy(w);
+}
+
 /***********************************************************************/
 
 void vorbis_file_info_box(char *fn)
 {
-	gchar *track_name, *performer, *album_name, *date, *track_number;
-	gchar *genre, *user_comment, *tmp;
-	gchar *description, *version, *isrc, *copyright, *organization;
-	gchar *location;
-	gchar *rg_track_gain, *rg_album_gain, *rg_track_peak, *rg_album_peak;
+	char *track_name, *performer, *album_name, *date, *track_number;
+	char *genre, *user_comment, *tmp;
+	char *description, *version, *isrc, *copyright, *organization;
+	char *location, *vendor = "N/A";
+	char *rg_track_gain, *rg_album_gain, *rg_track_peak, *rg_album_peak;
 
-	gint time, minutes, seconds, bitrate, rate, channels, filesize, i;
+	int time, minutes, seconds, bitrate, avgbitrate, rate, channels;
+	int filesize, i;
 
 	OggVorbis_File vf;
 	vorbis_info *vi;
@@ -440,16 +451,12 @@ void vorbis_file_info_box(char *fn)
 	FILE *fh;
 	gboolean clear_vf = FALSE;
 
-	static GtkWidget *info_frame, *info_box, *bitrate_label, *rate_label;
-	static GtkWidget *channel_label, *length_label, *filesize_label;
-	static GtkWidget *replaygain_label, *audiophilegain_label, *peak_label;
-	static GtkWidget *filename_entry, *tag_frame;
-
 	g_free(vte.filename);
 	vte.filename = g_strdup(fn);
 	
 	if (!window)
 	{
+		GtkWidget *info_frame, *info_box;
 		GtkWidget *hbox, *label, *filename_hbox, *vbox, *left_vbox;
 		GtkWidget *table, *bbox, *cancel_button;
 		GtkWidget *save_button, *remove_button;
@@ -458,6 +465,8 @@ void vorbis_file_info_box(char *fn)
 		gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
 		gtk_signal_connect(GTK_OBJECT(window), "destroy", 
 			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &window);
+		gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
+				   keypress_cb, NULL);
 		gtk_container_set_border_width(GTK_CONTAINER(window), 10);
 
 		vbox = gtk_vbox_new(FALSE, 10);
@@ -724,6 +733,13 @@ void vorbis_file_info_box(char *fn)
 				      GTK_JUSTIFY_LEFT);
 		gtk_box_pack_start(GTK_BOX(info_box), bitrate_label, FALSE,
 				   FALSE, 0);
+
+		avgbitrate_label = gtk_label_new("");
+		gtk_misc_set_alignment(GTK_MISC(avgbitrate_label), 0, 0);
+		gtk_label_set_justify(GTK_LABEL(avgbitrate_label),
+				      GTK_JUSTIFY_LEFT);
+		gtk_box_pack_start(GTK_BOX(info_box), avgbitrate_label, FALSE,
+				   FALSE, 0);
 		
 		rate_label = gtk_label_new("");
 		gtk_misc_set_alignment(GTK_MISC(rate_label), 0, 0);
@@ -751,6 +767,13 @@ void vorbis_file_info_box(char *fn)
 				      GTK_JUSTIFY_LEFT);
 		gtk_box_pack_start(GTK_BOX(info_box), filesize_label, FALSE,
 				   FALSE, 0);
+	
+		vendor_label = gtk_label_new("");
+		gtk_misc_set_alignment(GTK_MISC(vendor_label), 0, 0);
+		gtk_label_set_justify(GTK_LABEL(vendor_label),
+				      GTK_JUSTIFY_LEFT);
+		gtk_box_pack_start(GTK_BOX(info_box), vendor_label, FALSE,
+				   FALSE, 0);
 
 		replaygain_label = gtk_label_new("");
 		gtk_misc_set_alignment(GTK_MISC(replaygain_label), 0, 0);
@@ -783,10 +806,12 @@ void vorbis_file_info_box(char *fn)
 		gtk_widget_set_sensitive(tag_frame, TRUE);		
 
 	gtk_label_set_text(GTK_LABEL(bitrate_label), "");
+	gtk_label_set_text(GTK_LABEL(avgbitrate_label), "");
 	gtk_label_set_text(GTK_LABEL(rate_label), "");
 	gtk_label_set_text(GTK_LABEL(channel_label), "");
 	gtk_label_set_text(GTK_LABEL(length_label), "");
 	gtk_label_set_text(GTK_LABEL(filesize_label), "");
+	gtk_label_set_text(GTK_LABEL(vendor_label), "");
 
 	if ((fh = fopen(vte.filename, "r")) != NULL)
 	{
@@ -795,17 +820,24 @@ void vorbis_file_info_box(char *fn)
 		if (ov_open(fh, &vf, NULL, 0) == 0)
 		{
 			comment = ov_comment(&vf, -1);
+			if (comment && comment->vendor)
+				vendor = comment->vendor;
+
 			if ((vi = ov_info(&vf, 0)) != NULL)
 			{
 				bitrate = vi->bitrate_nominal/1000;
+				avgbitrate = ov_bitrate(&vf, -1);
+				if (avgbitrate == OV_EINVAL ||
+				    avgbitrate == OV_FALSE)
+					avgbitrate = 0;
 				rate = vi->rate;
 				channels = vi->channels;
-				g_message("version: %d", vi->version);
 				clear_vf = TRUE;
 			}
 			else
 			{
 				bitrate = 0;
+				avgbitrate = 0;
 				rate = 0;
 				channels = 0;
 			}
@@ -817,11 +849,12 @@ void vorbis_file_info_box(char *fn)
 			filesize = ftell(fh);
 
 			label_set_text(bitrate_label, _("Nominal bitrate: %d kbps"), bitrate);
+			label_set_text(avgbitrate_label, _("Average bitrate: %.1f kbps"), ((float) avgbitrate) / 1000);
 			label_set_text(rate_label, _("Samplerate: %d Hz"), rate);
 			label_set_text(channel_label, _("Channels: %d"), channels);
 			label_set_text(length_label, _("Length: %d:%.2d"), minutes, seconds);
 			label_set_text(filesize_label, _("File size: %d B"), filesize);
-
+			label_set_text(vendor_label, _("Vendor: %s"), vendor);
 		}
 		else
 			fclose(fh);
@@ -925,7 +958,3 @@ void vorbis_file_info_box(char *fn)
 	g_free(rg_track_peak);
 	g_free(rg_album_peak);
 }
-
-static void vorbis_advanced_tag_edit()
-{
-}
