@@ -47,6 +47,7 @@
 #include "libxmms/util.h"
 #include "libxmms/configfile.h"
 #include "libxmms/titlestring.h"
+#include "libxmms/charset.h"
 #include <xmms/i18n.h>
 
 #include "vorbis.h"
@@ -112,10 +113,12 @@ InputPlugin vorbis_ip =
 static OggVorbis_File vf;
 
 static pthread_t tid;
-int vorbis_playing = 0;
-static int vorbis_eos = 0;
-static int vorbis_is_streaming = 0;
-static int vorbis_bytes_streamed = 0;
+int vorbis_playing;
+static int output_opened;
+static int dopause;
+static int vorbis_eos;
+static int vorbis_is_streaming;
+static int vorbis_bytes_streamed;
 static volatile int seekneeded = -1;
 static int samplerate, channels;
 pthread_mutex_t vf_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -384,6 +387,11 @@ static void *vorbis_play_loop(void *arg)
 		goto play_cleanup;
 	}
 	pthread_mutex_unlock(&vf_mutex);
+	output_opened = TRUE;
+	if (dopause) {
+		vorbis_pause(TRUE);
+		dopause = FALSE;
+	}
 
 	seekneeded = -1;
    
@@ -455,6 +463,7 @@ static void *vorbis_play_loop(void *arg)
 			timercount = vorbis_ip.output->output_time();
 		}
 	}
+	output_opened = FALSE;
 	if (!output_error)
 		vorbis_ip.output->close_audio();
 	/* fall through intentional */
@@ -480,7 +489,9 @@ static void vorbis_play(char *filename)
 	vorbis_playing = 1;
 	vorbis_bytes_streamed = 0;
 	vorbis_eos = 0;
+	output_opened = FALSE;
 	output_error = FALSE;
+	dopause = FALSE;
 	pthread_create(&tid, NULL, vorbis_play_loop, g_strdup(filename));
 }
  
@@ -495,7 +506,10 @@ static void vorbis_stop(void)
 
 static void vorbis_pause(short p)
 {
-	vorbis_ip.output->pause(p);
+	if (output_opened)
+		vorbis_ip.output->pause(p);
+	else
+		dopause = p;
 }
 
 static int vorbis_time(void)
@@ -627,7 +641,7 @@ static gboolean vorbis_update_replaygain(float *scale)
 			*scale = 1.0 / rg_peak;
 	}
 	
-	if (*scale != 1.0 || vorbis_cfg.use_booster) {
+	if (*scale != 1.0 || (vorbis_cfg.use_booster && rg_gain)) {
 		if (*scale > 15.0)
 			*scale = 15.0;
 
@@ -689,17 +703,23 @@ static char *vorbis_generate_title(OggVorbis_File *vorbisfile, char *fn)
 
 	if ((comment = ov_comment(vorbisfile, -1)) != NULL)
 	{
-		input->track_name = convert_from_utf8(vorbis_comment_query(comment, "title", 0));
-		input->performer = convert_from_utf8(vorbis_comment_query(comment, "artist", 0));
-		input->album_name = convert_from_utf8(vorbis_comment_query(comment, "album", 0));
+		input->track_name = xmms_charset_from_utf8(
+			vorbis_comment_query(comment, "title", 0));
+		input->performer = xmms_charset_from_utf8(
+			vorbis_comment_query(comment, "artist", 0));
+		input->album_name = xmms_charset_from_utf8(
+			vorbis_comment_query(comment, "album", 0));
 		if ((tmp = vorbis_comment_query(comment, "tracknumber", 0)) != NULL)
 			input->track_number = atoi(tmp);
-		input->date = convert_from_utf8(vorbis_comment_query(comment, "date", 0));
+		input->date = xmms_charset_from_utf8(
+			vorbis_comment_query(comment, "date", 0));
 		if ((tmp = vorbis_comment_query(comment, "year", 0)) != NULL)
 			input->year = atoi(tmp);
-		input->genre = convert_from_utf8(vorbis_comment_query(comment, "genre", 0));
+		input->genre = xmms_charset_from_utf8(
+			vorbis_comment_query(comment, "genre", 0));
 		/* "comment" = user comment, not "" */
-		input->comment = convert_from_utf8(vorbis_comment_query(comment, "comment", 0));
+		input->comment = xmms_charset_from_utf8(
+			vorbis_comment_query(comment, "comment", 0));
 	}
 
 	displaytitle = xmms_get_titlestring(vorbis_cfg.tag_override ?
@@ -755,7 +775,7 @@ static void vorbis_aboutbox()
 		  "Haavard Kvaalen <havardk@xmms.org>\n"
 		  "Gian-Carlo Pascutto <gcp@sjeng.org>\n\n"
 		  "Visit the Xiph.org Foundation at http://www.xiph.org/\n"),
-		_("Ok"), FALSE, NULL, NULL);
+		_("OK"), FALSE, NULL, NULL);
 	gtk_signal_connect(GTK_OBJECT(about_window), "destroy",
 			   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
 			   &about_window);
