@@ -5,9 +5,6 @@ void vorbis_configure(void);
 
 void vorbis_file_info_box(char *filename);
 
-char* convert_to_utf8(const char *string);
-char* convert_from_utf8(const char *string);
-
 typedef struct {
 	gint http_buffer_size;
 	gint http_prebuffer;
