@@ -2,7 +2,8 @@
  *  Copyright (C) 2001-2003 Matthieu Sozeau <mattam@altern.org>
  *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2004  Haavard Kvaalen
+ *  Copyright (C) 1999-2006  Haavard Kvaalen
+ *  Copyright (C) 2005       Takashi Iwai
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,34 +20,62 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/*
+ *  CHANGES
+ *
+ *  2005.01.05  Takashi Iwai <tiwai@suse.de>
+ *	Impelemented the multi-threaded mode with an audio-thread.
+ *	Many fixes and cleanups.
+ */
+
+
 #include "alsa.h"
 #include <ctype.h>
+#include <pthread.h>
+#include <math.h>
 #include <libxmms/xconvert.h>
 
-static snd_pcm_t		*alsa_pcm 	    = NULL;
-static snd_pcm_status_t		*alsa_status 	    = NULL;
-static snd_pcm_channel_area_t   *areas              = NULL;
+static snd_pcm_t *alsa_pcm;
+static snd_output_t *logs;
 
-static snd_output_t		*logs 		    = NULL;
 
-static int 			 alsa_bps 	    = 0;
-static guint64			 alsa_total_written = 0;
+/* Number of bytes that we have received from the input plugin */
+static guint64 alsa_total_written;
+/* Number of bytes that we have sent to the sound card */
+static guint64 alsa_hw_written;
+static guint64 output_time_offset;
+
+/* device buffer/period sizes in bytes */
+static int hw_buffer_size, hw_period_size;		/* in output bytes */
+static int hw_buffer_size_in, hw_period_size_in;	/* in input bytes */
 
 /* Set/Get volume */
-static snd_mixer_elem_t 	*pcm_element 	    = NULL;
-static snd_mixer_t 		*mixer 		    = NULL;
+static snd_mixer_elem_t *pcm_element;
+static snd_mixer_t *mixer;
 
-static gboolean mmap, force_start, going, paused;
+static gboolean going, paused, mixer_start = TRUE;
+static gboolean prebuffer, remove_prebuffer;
 
-static gpointer buffer;
+static gboolean alsa_can_pause;
 
-static int alsa_can_pause;
+/* for audio thread */
+static pthread_t audio_thread;	 /* audio loop thread */
+static int thread_buffer_size;	 /* size of intermediate buffer in bytes */
+static char *thread_buffer;	 /* audio intermediate buffer */
+static int rd_index, wr_index;	 /* current read/write position in int-buffer */
+static gboolean pause_request;	 /* pause status currently requested */
+static int flush_request;	 /* flush status (time) currently requested */
+static int prebuffer_size;
+
+static guint mixer_timeout;
 
 struct snd_format {
 	unsigned int rate;
 	unsigned int channels;
 	snd_pcm_format_t format;
 	AFormat xmms_format;
+	int sample_bits;
+	int bps;
 };
 
 static struct snd_format *inputf = NULL;
@@ -54,8 +83,9 @@ static struct snd_format *effectf = NULL;
 static struct snd_format *outputf = NULL;
 
 static int alsa_setup(struct snd_format *f);
-static void alsa_mmap_audio(char *data, int length);
-static void alsa_write_audio(gpointer data, int length);
+static void alsa_write_audio(char *data, int length);
+static void alsa_cleanup_mixer(void);
+static int get_thread_buffer_filled(void);
 
 static struct snd_format * snd_format_from_xmms(AFormat fmt, int rate, int channels);
 
@@ -71,22 +101,10 @@ static const struct {
 } format_table[] =
 {{FMT_S16_LE, SND_PCM_FORMAT_S16_LE},
  {FMT_S16_BE, SND_PCM_FORMAT_S16_BE},
- {FMT_S16_NE,
-#ifdef WORDS_BIGENDIAN
-  SND_PCM_FORMAT_S16_BE
-#else
-  SND_PCM_FORMAT_S16_LE
-#endif
- },
+ {FMT_S16_NE, SND_PCM_FORMAT_S16},
  {FMT_U16_LE, SND_PCM_FORMAT_U16_LE},
  {FMT_U16_BE, SND_PCM_FORMAT_U16_BE},
- {FMT_U16_NE,
-#ifdef WORDS_BIGENDIAN
-  SND_PCM_FORMAT_U16_BE
-#else
-  SND_PCM_FORMAT_U16_LE
-#endif
- },
+ {FMT_U16_NE, SND_PCM_FORMAT_U16},
  {FMT_U8, SND_PCM_FORMAT_U8},
  {FMT_S8, SND_PCM_FORMAT_S8},
 };
@@ -108,20 +126,19 @@ static void debug(char *str, ...)
 
 int alsa_playing(void)
 {
-	if (!going || paused)
+	if (!going || paused || alsa_pcm == NULL)
 		return FALSE;
 
-	return(snd_pcm_state(alsa_pcm) == SND_PCM_STATE_RUNNING);
+	return snd_pcm_state(alsa_pcm) == SND_PCM_STATE_RUNNING;
 }
 
-static void xrun_recover(void)
+static int xrun_recover(void)
 {
-	int err;
-
 	if (alsa_cfg.debug)
 	{
+		snd_pcm_status_t *alsa_status;
 		snd_pcm_status_alloca(&alsa_status);
-		if ((err = snd_pcm_status(alsa_pcm, alsa_status)) < 0)
+		if (snd_pcm_status(alsa_pcm, alsa_status) < 0)
 			g_warning("xrun_recover(): snd_pcm_status() failed");
 		else
 		{
@@ -129,110 +146,125 @@ static void xrun_recover(void)
 			snd_pcm_status_dump(alsa_status, logs);
 		}
 	}
-
-	if (snd_pcm_state(alsa_pcm) == SND_PCM_STATE_XRUN)
-	{
-		if ((err = snd_pcm_prepare(alsa_pcm)) < 0)
-			g_warning("xrun_recover(): snd_pcm_prepare() failed.");
-	}
+	return snd_pcm_prepare(alsa_pcm);
 }
 
+static int suspend_recover(void)
+{
+	int err;
+
+	while ((err = snd_pcm_resume(alsa_pcm)) == -EAGAIN)
+		/* wait until suspend flag is released */
+		sleep(1);
+	if (err < 0)
+	{
+		g_warning("alsa_handle_error(): "
+			  "snd_pcm_resume() failed.");
+		return snd_pcm_prepare(alsa_pcm);
+	}
+	return err;
+}
+
+/* handle generic errors */
+static int alsa_handle_error(int err)
+{
+	switch (err)
+	{
+		case -EPIPE:
+			return xrun_recover();
+		case -ESTRPIPE:
+			return suspend_recover();
+	}
+
+	return err;
+}
+
+/* update and get the available space on h/w buffer (in frames) */
 static snd_pcm_sframes_t alsa_get_avail(void)
 {
 	snd_pcm_sframes_t ret;
-	if ((ret = snd_pcm_avail_update(alsa_pcm)) == -EPIPE)
-		xrun_recover();
-	else if (ret < 0)
-	{
-		g_warning("alsa_get_avail(): snd_pcm_avail_update() failed: %s",
-			  snd_strerror(-ret));
+
+	if (alsa_pcm == NULL)
 		return 0;
-	}
-	else
-		return ret;
-	if ((ret = snd_pcm_avail_update(alsa_pcm)) < 0)
+
+	while ((ret = snd_pcm_avail_update(alsa_pcm)) < 0)
 	{
-		g_warning("alsa_get_avail(): snd_pcm_avail_update() failed: %s",
-			  snd_strerror(-ret));
-		return 0;
+		ret = alsa_handle_error(ret);
+		if (ret < 0)
+		{
+			g_warning("alsa_get_avail(): snd_pcm_avail_update() failed: %s",
+				  snd_strerror(-ret));
+			return 0;
+		}
 	}
 	return ret;
 }
 
+/* get the free space on buffer */
 int alsa_free(void)
 {
-	if (paused)
-		return 0;
-	else
+	if (remove_prebuffer && prebuffer)
 	{
-		int err;
-		if (force_start &&
-		    snd_pcm_state(alsa_pcm) == SND_PCM_STATE_PREPARED)
-		{
-			if ((err = snd_pcm_start(alsa_pcm)) < 0)
-				g_warning("alsa_free(): snd_pcm_start() "
-					  "failed: %s", snd_strerror(-err));
-			else
-				debug("Stream started");
-		}
-		force_start = TRUE;
-
-		return snd_pcm_frames_to_bytes(alsa_pcm, alsa_get_avail());
+		prebuffer = FALSE;
+		remove_prebuffer = FALSE;
 	}
+	if (prebuffer)
+		remove_prebuffer = TRUE;
+	
+	return thread_buffer_size - get_thread_buffer_filled() - 1;
+}
+
+/* do pause operation */
+static void alsa_do_pause(gboolean p)
+{
+	if (paused == p)
+		return;
+
+	if (alsa_pcm)
+	{
+		if (alsa_can_pause)
+			snd_pcm_pause(alsa_pcm, p);
+		else if (p)
+		{
+			snd_pcm_drop(alsa_pcm);
+			snd_pcm_prepare(alsa_pcm);
+		}
+	}
+	paused = p;
 }
 
 void alsa_pause(short p)
 {
 	debug("alsa_pause");
-	if (p)
-		paused = TRUE;
-
-	if (alsa_can_pause)
-		snd_pcm_pause(alsa_pcm, p);
-	else if (p)
-		snd_pcm_drop(alsa_pcm);
-
-	if (!p)
-		paused = FALSE;
+	pause_request = p;
 }
 
-void alsa_close(void)
+/* close PCM and release associated resources */
+static void alsa_close_pcm(void)
 {
-	int err, started;
-
-	debug("Closing device");
-
-	started = going;
-	going = 0;
-
-	pcm_element = NULL;
-
-	if (mixer)
+	if (alsa_pcm)
 	{
-		snd_mixer_close(mixer);
-		mixer = NULL;
-	}
-
-	if (alsa_pcm != NULL)
-	{
-		if (started)
-			if ((err = snd_pcm_drop(alsa_pcm)) < 0)
-				g_warning("alsa_pcm_drop() failed: %s",
-					  snd_strerror(-err));
-
+		int err;
+		snd_pcm_drop(alsa_pcm);
 		if ((err = snd_pcm_close(alsa_pcm)) < 0)
 			g_warning("alsa_pcm_close() failed: %s",
 				  snd_strerror(-err));
 		alsa_pcm = NULL;
 	}
+}
 
-	if (mmap) {
-		g_free(buffer);
-		buffer = NULL;
+void alsa_close(void)
+{
+	if (!going)
+		return;
 
-		g_free(areas);
-		areas = NULL;
-	}
+	debug("Closing device");
+
+	going = 0;
+
+	pthread_join(audio_thread, NULL);
+
+	alsa_cleanup_mixer();
 
 	xmms_convert_buffers_destroy(convertb);
 	convertb = NULL;
@@ -240,40 +272,47 @@ void alsa_close(void)
 	inputf = NULL;
 	g_free(effectf);
 	effectf = NULL;
+	g_free(outputf);
+	outputf = NULL;
 
 	alsa_save_config();
 
+	if (alsa_cfg.debug)
+		snd_output_close(logs);
 	debug("Device closed");
 }
 
-static void alsa_reopen(struct snd_format *f)
+/* reopen ALSA PCM */
+static int alsa_reopen(struct snd_format *f)
 {
-	unsigned int tmp = alsa_get_written_time();
+	/* remember the current position */
+	output_time_offset += (alsa_hw_written * 1000) / outputf->bps;
+	alsa_hw_written = 0;
 
-	if (alsa_pcm != NULL)
+	alsa_close_pcm();
+
+	return alsa_setup(f);
+}
+
+/* do flush (drop) operation */
+static void alsa_do_flush(int time)
+{
+	if (alsa_pcm)
 	{
-		snd_pcm_close(alsa_pcm);
-		alsa_pcm = NULL;
+		snd_pcm_drop(alsa_pcm);
+		snd_pcm_prepare(alsa_pcm);
 	}
-
-	if (mmap) {
-		g_free(buffer);
-		buffer = NULL;
-		
-		g_free(areas);
-		areas = NULL;
-	}
-
-	if (alsa_setup(f) < 0)
-		g_warning("Failed to reopen the audio device");
-
-	alsa_total_written = tmp;
-	snd_pcm_prepare(alsa_pcm);
+	/* correct the offset */
+	output_time_offset = time;
+	alsa_total_written = (guint64) time * inputf->bps / 1000;
+	rd_index = wr_index = alsa_hw_written = 0;
 }
 
 void alsa_flush(int time)
 {
-	alsa_total_written = (guint64) time * alsa_bps / 1000;
+	flush_request = time;
+	while (flush_request != -1)
+		xmms_usleep(10000);
 }
 
 static void parse_mixer_name(char *str, char **name, int *index)
@@ -296,14 +335,12 @@ static void parse_mixer_name(char *str, char **name, int *index)
 	}
 }
 
-int alsa_get_mixer(snd_mixer_t **mixer, int card) 
+int alsa_get_mixer(snd_mixer_t **mixer, int card)
 {
 	char *dev;
 	int err;
-	
+
 	debug("alsa_get_mixer");
-	
-	dev = g_strdup_printf("hw:%i", card);
 
 	if ((err = snd_mixer_open(mixer, 0)) < 0)
 	{
@@ -312,12 +349,17 @@ int alsa_get_mixer(snd_mixer_t **mixer, int card)
 		mixer = NULL;
 		return -1;
 	}
+
+	dev = g_strdup_printf("hw:%i", card);
 	if ((err = snd_mixer_attach(*mixer, dev)) < 0)
 	{
 		g_warning("alsa_get_mixer(): Attaching to mixer %s failed: %s",
 			  dev, snd_strerror(-err));
+		g_free(dev);
 		return -1;
 	}
+	g_free(dev);
+
 	if ((err = snd_mixer_selem_register(*mixer, NULL, NULL)) < 0)
 	{
 		g_warning("alsa_get_mixer(): Failed to register mixer: %s",
@@ -330,14 +372,12 @@ int alsa_get_mixer(snd_mixer_t **mixer, int card)
 			  snd_strerror(-err));
 		return -1;
 	}
-	
-	g_free(dev);
 
 	return (*mixer != NULL);
 }
-	
 
-snd_mixer_elem_t* alsa_get_mixer_elem(snd_mixer_t *mixer, char *name, int index)
+
+static snd_mixer_elem_t* alsa_get_mixer_elem(snd_mixer_t *mixer, char *name, int index)
 {
 	snd_mixer_selem_id_t *selem_id;
 	snd_mixer_elem_t* elem;
@@ -349,11 +389,11 @@ snd_mixer_elem_t* alsa_get_mixer_elem(snd_mixer_t *mixer, char *name, int index)
 		snd_mixer_selem_id_set_name(selem_id, name);
 
 	elem = snd_mixer_find_selem(mixer, selem_id);
-	
+
 	return elem;
 }
 
-int alsa_setup_mixer(void)
+static int alsa_setup_mixer(void)
 {
 	char *name;
 	long int a, b;
@@ -361,16 +401,16 @@ int alsa_setup_mixer(void)
 	int err, index;
 
 	debug("alsa_setup_mixer");
-	
+
 	if ((err = alsa_get_mixer(&mixer, alsa_cfg.mixer_card)) < 0)
 		return err;
 
 	parse_mixer_name(alsa_cfg.mixer_device, &name, &index);
-	
+
 	pcm_element = alsa_get_mixer_elem(mixer, name, index);
-	
+
 	g_free(name);
-	
+
 	if (!pcm_element)
 	{
 		g_warning("alsa_setup_mixer(): Failed to find mixer element: %s",
@@ -406,15 +446,44 @@ int alsa_setup_mixer(void)
 	return 0;
 }
 
+static int alsa_mixer_timeout(void *data)
+{
+	if (mixer)
+	{
+		snd_mixer_close(mixer);
+		mixer = NULL;
+		pcm_element = NULL;
+	}
+	mixer_timeout = 0;
+	mixer_start = TRUE;
+
+	return FALSE;
+}
+
+static void alsa_cleanup_mixer(void)
+{
+	pcm_element = NULL;
+	if (mixer)
+	{
+		snd_mixer_close(mixer);
+		mixer = NULL;
+	}
+}
+
 void alsa_get_volume(int *l, int *r)
 {
-	static gboolean first = TRUE;
 	long ll = *l, lr = *r;
 
-	if (first)
+	if (mixer_start)
 	{
 		alsa_setup_mixer();
-		first = !first;
+		mixer_start = FALSE;
+	}
+
+	if (alsa_cfg.soft_volume)
+	{
+		*l = alsa_cfg.vol.left;
+		*r = alsa_cfg.vol.right;
 	}
 
 	if (!pcm_element)
@@ -422,12 +491,7 @@ void alsa_get_volume(int *l, int *r)
 
 	snd_mixer_handle_events(mixer);
 
-	if (alsa_cfg.soft_volume)
-	{
-		*l = alsa_cfg.vol.left;
-		*r = alsa_cfg.vol.right;
-	}
-	else
+	if (!alsa_cfg.soft_volume)
 	{
 		snd_mixer_selem_get_playback_volume(pcm_element,
 						    SND_MIXER_SCHN_FRONT_LEFT,
@@ -438,48 +502,67 @@ void alsa_get_volume(int *l, int *r)
 		*l = ll;
 		*r = lr;
 	}
+	if (mixer_timeout)
+		gtk_timeout_remove(mixer_timeout);
+	mixer_timeout = gtk_timeout_add(5000, alsa_mixer_timeout, NULL);
 }
 
 
 void alsa_set_volume(int l, int r)
 {
-	if (!pcm_element)
-		return;
-
 	if (alsa_cfg.soft_volume)
 	{
 		alsa_cfg.vol.left = l;
 		alsa_cfg.vol.right = r;
+		return;
 	}
-	else
-	{
-		snd_mixer_selem_set_playback_volume(pcm_element,
-						    SND_MIXER_SCHN_FRONT_LEFT, l);
-		snd_mixer_selem_set_playback_volume(pcm_element,
-						    SND_MIXER_SCHN_FRONT_RIGHT, r);
-	}
+
+	if (!pcm_element)
+		return;
+
+	snd_mixer_selem_set_playback_volume(pcm_element,
+					    SND_MIXER_SCHN_FRONT_LEFT, l);
+	snd_mixer_selem_set_playback_volume(pcm_element,
+					    SND_MIXER_SCHN_FRONT_RIGHT, r);
 }
 
 
+/*
+ * audio stuff
+ */
+
+/* return the size of audio data filled in the audio thread buffer */
+static int get_thread_buffer_filled(void)
+{
+	if (wr_index >= rd_index)
+		return wr_index - rd_index;
+	return thread_buffer_size - (rd_index - wr_index);
+}
+
 int alsa_get_output_time(void)
 {
 	snd_pcm_sframes_t delay;
-	ssize_t db = 0;
+	guint64 bytes = alsa_hw_written;
 
-	if (!going)
+	if (!going || alsa_pcm == NULL)
 		return 0;
 
 	if (!snd_pcm_delay(alsa_pcm, &delay))
-		db = snd_pcm_frames_to_bytes(alsa_pcm, delay);
-	
-	if (db < alsa_total_written)
-		return ((alsa_total_written - db) * 1000 / alsa_bps);
-	return 0;
+	{
+		unsigned int d = snd_pcm_frames_to_bytes(alsa_pcm, delay);
+		if (bytes < d)
+			bytes = 0;
+		else
+			bytes -= d;
+	}
+	return output_time_offset + (bytes * 1000) / outputf->bps;
 }
 
 int alsa_get_written_time(void)
 {
-	return (alsa_total_written * 1000 / alsa_bps);
+	if (!going)
+		return 0;
+	return (alsa_total_written * 1000) / inputf->bps;
 }
 
 #define STEREO_ADJUST(type, type2, endian)					\
@@ -488,10 +571,10 @@ do {										\
 	for (i = 0; i < length; i += 4)						\
 	{									\
 		*ptr = type2##_TO_##endian(type2##_FROM_## endian(*ptr) *	\
-					   alsa_cfg.vol.left / 100);		\
+					   lvol / 256);				\
 		ptr++;								\
 		*ptr = type2##_TO_##endian(type2##_FROM_##endian(*ptr) *	\
-					   alsa_cfg.vol.right / 100);		\
+					   rvol / 256);				\
 		ptr++;								\
 	}									\
 } while (0)
@@ -499,10 +582,10 @@ do {										\
 #define MONO_ADJUST(type, type2, endian)					\
 do {										\
 	type *ptr = data;							\
-	for (i = 0; i < length; i += 4)						\
+	for (i = 0; i < length; i += 2)						\
 	{									\
 		*ptr = type2##_TO_##endian(type2##_FROM_## endian(*ptr) *	\
-					   vol / 100);				\
+					   vol / 256);				\
 		ptr++;								\
 	}									\
 } while (0)
@@ -513,16 +596,16 @@ do {							\
 		STEREO_ADJUST(type, type2, endian);	\
 	else						\
 		MONO_ADJUST(type, type2, endian);	\
-} while (0)	
+} while (0)
 
 #define STEREO_ADJUST8(type)				\
 do {							\
 	type *ptr = data;				\
 	for (i = 0; i < length; i += 2)			\
 	{						\
-		*ptr = *ptr * alsa_cfg.vol.left / 100;	\
+		*ptr = *ptr * lvol / 256;		\
 		ptr++;					\
-		*ptr = *ptr * alsa_cfg.vol.right / 100;	\
+		*ptr = *ptr * rvol / 256;		\
 		ptr++;					\
 	}						\
 } while (0)
@@ -530,9 +613,9 @@ do {							\
 #define MONO_ADJUST8(type)			\
 do {						\
 	type *ptr = data;			\
-	for (i = 0; i < length; i += 4)		\
+	for (i = 0; i < length; i++)		\
 	{					\
-		*ptr = *ptr * vol / 100;	\
+		*ptr = *ptr * vol / 256;	\
 		ptr++;				\
 	}					\
 } while (0)
@@ -543,20 +626,22 @@ do {						\
 		STEREO_ADJUST8(type);		\
 	else					\
 		MONO_ADJUST8(type);		\
-} while (0)	
+} while (0)
 
 
 static void volume_adjust(void* data, int length, AFormat fmt, int channels)
 {
-	int i, vol;
+	int i, vol, lvol, rvol;
 
 	if ((alsa_cfg.vol.left == 100 && alsa_cfg.vol.right == 100) ||
 	    (channels == 1 &&
 	     (alsa_cfg.vol.left == 100 || alsa_cfg.vol.right == 100)))
 		return;
 
-	vol = MAX(alsa_cfg.vol.left, alsa_cfg.vol.right);
-	
+	lvol = pow(10, (alsa_cfg.vol.left - 100) / 40.0) * 256;
+	rvol = pow(10, (alsa_cfg.vol.right - 100) / 40.0) * 256;
+	vol = MAX(lvol, rvol);
+
 	switch (fmt)
 	{
 		case FMT_S16_LE:
@@ -584,53 +669,48 @@ static void volume_adjust(void* data, int length, AFormat fmt, int channels)
 }
 
 
-void alsa_write(gpointer data, int length)
+/* transfer data to audio h/w; length is given in bytes
+ *
+ * data can be modified via effect plugin, rate conversion or
+ * software volume before passed to audio h/w
+ */
+static void alsa_do_write(gpointer data, int length)
 {
-	EffectPlugin *ep;
+	EffectPlugin *ep = NULL;
+	int new_freq;
+	int new_chn;
+	AFormat f;
 
 	if (paused)
 		return;
 
-	force_start = FALSE;
+	new_freq = inputf->rate;
+	new_chn = inputf->channels;
+	f = inputf->xmms_format;
 
-	if (effects_enabled() && (ep = get_current_effect_plugin()))
+	if (effects_enabled() && (ep = get_current_effect_plugin()) &&
+	    ep->query_format)
+		ep->query_format(&f, &new_freq, &new_chn);
+
+	if (f != effectf->xmms_format || new_freq != effectf->rate ||
+	    new_chn != effectf->channels)
 	{
-		int new_freq = inputf->rate;
-		int new_chn = inputf->channels;
-		AFormat f = inputf->xmms_format;
-		
-		if (ep->query_format)
+		debug("Changing audio format for effect plugin");
+		g_free(effectf);
+		effectf = snd_format_from_xmms(f, new_freq, new_chn);
+		if (alsa_reopen(effectf) < 0)
 		{
-			ep->query_format(&f, &new_freq, &new_chn);
-			
-			if (f != effectf->xmms_format ||
-			    new_freq != effectf->rate ||
-			    new_chn != effectf->channels)
-			{
-				debug("Changing audio format for effect plugin");
-
-				g_free(effectf);
-				effectf = snd_format_from_xmms(f, new_freq,
-							       new_chn);
-				alsa_reopen(effectf);
-			}
-			
+			/* fatal error... */
+			alsa_close();
+			return;
 		}
-	
+	}
+
+	if (ep)
 		length = ep->mod_samples(&data, length,
 					 inputf->xmms_format,
 					 inputf->rate,
 					 inputf->channels);
-	}
-	else if (effectf)
-	{
-		g_free(effectf);
-		effectf = NULL;
-		effectf = snd_format_from_xmms(inputf->xmms_format,
-					       inputf->rate,
-					       inputf->channels);
-		alsa_reopen(inputf);
-	}
 
 	if (alsa_convert_func != NULL)
 		length = alsa_convert_func(convertb, &data, length);
@@ -644,13 +724,32 @@ void alsa_write(gpointer data, int length)
 	if (alsa_cfg.soft_volume)
 		volume_adjust(data, length, outputf->xmms_format, outputf->channels);
 
-	if (mmap)
-		alsa_mmap_audio(data, length);
-	else
-		alsa_write_audio(data, length);
+	alsa_write_audio(data, length);
 }
 
-static void alsa_write_audio(gpointer data, int length)
+/* write callback */
+void alsa_write(gpointer data, int length)
+{
+	int cnt;
+	char *src = (char *)data;
+	
+	remove_prebuffer = FALSE;
+	
+	alsa_total_written += length;
+	while (length > 0)
+	{
+		int wr;
+		cnt = MIN(length, thread_buffer_size - wr_index);
+		memcpy(thread_buffer + wr_index, src, cnt);
+		wr = (wr_index + cnt) % thread_buffer_size;
+		wr_index = wr;
+		length -= cnt;
+		src += cnt;
+	}
+}
+
+/* transfer data to audio h/w via normal write */
+static void alsa_write_audio(char *data, int length)
 {
 	snd_pcm_sframes_t written_frames;
 
@@ -658,106 +757,150 @@ static void alsa_write_audio(gpointer data, int length)
 	{
 		int frames = snd_pcm_bytes_to_frames(alsa_pcm, length);
 		written_frames = snd_pcm_writei(alsa_pcm, data, frames);
-		
+
 		if (written_frames > 0)
 		{
 			int written = snd_pcm_frames_to_bytes(alsa_pcm,
 							      written_frames);
-			alsa_total_written += written;
 			length -= written;
-			data = (char*) data + written;
+			data += written;
+			alsa_hw_written += written;
 		}
-		else if (written_frames == -EPIPE)
-			xrun_recover();
 		else
 		{
-			g_warning("alsa_write_audio(): write error: %s",
-				  snd_strerror(-written_frames));
-			break;
+			int err = alsa_handle_error((int)written_frames);
+			if (err < 0)
+			{
+				g_warning("alsa_write_audio(): write error: %s",
+					  snd_strerror(-err));
+				break;
+			}
 		}
 	}
 }
 
-static void alsa_mmap_audio(char *data, int length)
+/* transfer audio data from thread buffer to h/w */
+static void alsa_write_out_thread_data(void)
 {
-	int cnt = 0, err;
-	snd_pcm_uframes_t offset, frames, frame;
-	const snd_pcm_channel_area_t *chan_areas = areas;
-	int channel_offset = 0, channel;
-	ssize_t sample_size, offset_bytes, step;
-
-	alsa_get_avail();
+	gint length, cnt, avail;
 
+	length = MIN(hw_period_size_in, get_thread_buffer_filled());
+	avail = snd_pcm_frames_to_bytes(alsa_pcm, alsa_get_avail());
+	length = MIN(length, avail);
 	while (length > 0)
 	{
-		frames = snd_pcm_bytes_to_frames(alsa_pcm, length);
-		if ((err = snd_pcm_mmap_begin(alsa_pcm, &chan_areas, &offset, &frames) < 0))
-			g_warning("alsa_mmap_audio(): snd_pcm_mmap_begin() "
-				  "failed: %s", snd_strerror(-err));
-
-		cnt = snd_pcm_frames_to_bytes(alsa_pcm, frames);
-
-		sample_size = snd_pcm_samples_to_bytes(alsa_pcm, 1);
-		step = chan_areas[0].step / 8;
-		offset_bytes = offset * step;
-
-		for (frame = 0; frame < frames; frame++)
-		{
-			for (channel = 0; channel < outputf->channels; channel++)
-			{
-				char *ptr = chan_areas[channel].addr;
-				memcpy(ptr + chan_areas[channel].first / 8 +
-				       offset_bytes,
-				       data + channel_offset, sample_size);
-				channel_offset += sample_size;
-			}
-			offset_bytes += step;
-		}
-			
-		err = snd_pcm_mmap_commit(alsa_pcm, offset, frames);
-		if (err == -EPIPE)
-			xrun_recover();
-		else if (err < 0)
-			g_warning("alsa_mmap_audio(): snd_pcm_mmap_commit() "
-				  "failed: %s", snd_strerror(-err));
-		else if (err != frames)
-			g_warning("alsa_mmap_audio(): snd_pcm_mmap_commit "
-				  "returned %d, expected %d", err, (int)frames);
-		
-		alsa_total_written += cnt;
-		
+		int rd;
+		cnt = MIN(length, thread_buffer_size - rd_index);
+		alsa_do_write(thread_buffer + rd_index, cnt);
+		rd = (rd_index + cnt) % thread_buffer_size;
+		rd_index = rd;
 		length -= cnt;
 	}
 }
 
+/* audio thread loop */
+/* FIXME: proper lock? */
+static void *alsa_loop(void *arg)
+{
+	int npfds = snd_pcm_poll_descriptors_count(alsa_pcm);
+	struct pollfd *pfds;
+	unsigned short *revents;
+
+	if (npfds <= 0)
+		goto _error;
+	pfds = alloca(sizeof(*pfds) * npfds);
+	revents = alloca(sizeof(*revents) * npfds);
+	while (going && alsa_pcm)
+	{
+		if (get_thread_buffer_filled() > prebuffer_size)
+			prebuffer = FALSE;
+		if (!paused && !prebuffer &&
+		    get_thread_buffer_filled() > hw_period_size_in)
+		{
+			snd_pcm_poll_descriptors(alsa_pcm, pfds, npfds);
+			if (poll(pfds, npfds, 10) > 0)
+			{
+				/*
+				 * need to check revents.  poll() with
+				 * dmix returns a postive value even
+				 * if no data is available
+				 */
+				int i;
+				snd_pcm_poll_descriptors_revents(alsa_pcm, pfds,
+								 npfds, revents);
+				for (i = 0; i < npfds; i++)
+					if (revents[i] & POLLOUT)
+					{
+						alsa_write_out_thread_data();
+						break;
+					}
+			}
+		}
+		else
+			xmms_usleep(10000);
+
+		if (pause_request != paused)
+			alsa_do_pause(pause_request);
+
+		if (flush_request != -1)
+		{
+			alsa_do_flush(flush_request);
+			flush_request = -1;
+			prebuffer = TRUE;
+		}
+	}
+
+ _error:
+	alsa_close_pcm();
+	g_free(thread_buffer);
+	thread_buffer = NULL;
+	pthread_exit(NULL);
+}
+
+/* open callback */
 int alsa_open(AFormat fmt, int rate, int nch)
 {
 	debug("Opening device");
 	inputf = snd_format_from_xmms(fmt, rate, nch);
 	effectf = snd_format_from_xmms(fmt, rate, nch);
-	
+
 	if (alsa_cfg.debug)
 		snd_output_stdio_attach(&logs, stdout, 0);
 
-	mmap = alsa_cfg.mmap;
-
 	if (alsa_setup(inputf) < 0)
 	{
 		alsa_close();
 		return 0;
 	}
 
-	alsa_setup_mixer();
+	if (!mixer)
+		alsa_setup_mixer();
 
 	convertb = xmms_convert_buffers_new();
-	
-	alsa_total_written = 0;
+
+	output_time_offset = 0;
+	alsa_total_written = alsa_hw_written = 0;
 	going = TRUE;
 	paused = FALSE;
-	force_start = FALSE;
-	
-	snd_pcm_prepare(alsa_pcm);
+	prebuffer = TRUE;
+	remove_prebuffer = FALSE;
+
+	thread_buffer_size = (guint64)alsa_cfg.thread_buffer_time * inputf->bps / 1000;
+	if (thread_buffer_size < hw_buffer_size)
+		thread_buffer_size = hw_buffer_size * 2;
+	if (thread_buffer_size < 8192)
+		thread_buffer_size = 8192;
+	prebuffer_size = thread_buffer_size / 2;
+	if (prebuffer_size < 8192)
+		prebuffer_size = 8192;
+	thread_buffer_size += hw_buffer_size;
+	thread_buffer_size -= thread_buffer_size % hw_period_size;
+	thread_buffer = g_malloc0(thread_buffer_size);
+	wr_index = rd_index = 0;
+	pause_request = FALSE;
+	flush_request = -1;
 	
+	pthread_create(&audio_thread, NULL, alsa_loop, NULL);
 	return 1;
 }
 
@@ -787,6 +930,8 @@ static struct snd_format * snd_format_from_xmms(AFormat fmt, int rate, int chann
 
 	f->rate = rate;
 	f->channels = channels;
+	f->sample_bits = snd_pcm_format_physical_width(f->format);
+	f->bps = (rate * f->sample_bits * channels) >> 3;
 
 	return f;
 }
@@ -806,8 +951,7 @@ static int alsa_setup(struct snd_format *f)
 	int err;
 	snd_pcm_hw_params_t *hwparams;
 	snd_pcm_sw_params_t *swparams;
-	int alsa_buffer_time, bits_per_sample;
-	unsigned int alsa_period_time;
+	unsigned int alsa_buffer_time, alsa_period_time;
 	snd_pcm_uframes_t alsa_buffer_size, alsa_period_size;
 
 	debug("alsa_setup");
@@ -816,9 +960,8 @@ static int alsa_setup(struct snd_format *f)
 	alsa_stereo_convert_func = NULL;
 	alsa_frequency_convert_func = NULL;
 
-	outputf = snd_format_from_xmms(effectf->xmms_format,
-				       effectf->rate,
-				       effectf->channels);
+	g_free(outputf);
+	outputf = snd_format_from_xmms(f->xmms_format, f->rate, f->channels);
 
 	debug("Opening device: %s", alsa_cfg.pcm_device);
 	/* FIXME: Can snd_pcm_open() return EAGAIN? */
@@ -829,9 +972,13 @@ static int alsa_setup(struct snd_format *f)
 		g_warning("alsa_setup(): Failed to open pcm device (%s): %s",
 			  alsa_cfg.pcm_device, snd_strerror(-err));
 		alsa_pcm = NULL;
+		g_free(outputf);
+		outputf = NULL;
 		return -1;
 	}
-	snd_pcm_nonblock(alsa_pcm, FALSE);
+
+	/* doesn't care about non-blocking */
+	/* snd_pcm_nonblock(alsa_pcm, 0); */
 
 	if (alsa_cfg.debug)
 	{
@@ -856,17 +1003,7 @@ static int alsa_setup(struct snd_format *f)
 		return -1;
 	}
 
-	if (mmap &&
-	    (err = snd_pcm_hw_params_set_access(alsa_pcm, hwparams,
-						SND_PCM_ACCESS_MMAP_INTERLEAVED)) < 0)
-	{
-		g_message("alsa_setup(): Cannot set mmap'ed mode: %s. "
-			  "falling back to direct write", snd_strerror(-err));
-		mmap = 0;
-	}
-
-	if (!mmap &&
-	    (err = snd_pcm_hw_params_set_access(alsa_pcm, hwparams,
+	if ((err = snd_pcm_hw_params_set_access(alsa_pcm, hwparams,
 						SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
 	{
 		g_warning("alsa_setup(): Cannot set direct write mode: %s",
@@ -894,17 +1031,17 @@ static int alsa_setup(struct snd_format *f)
 				break;
 			}
 		}
-		if (outputf->format != effectf->format)
+		if (outputf->format != f->format)
 		{
 			outputf->xmms_format =
 				format_from_alsa(outputf->format);
 			debug("Converting format from %d to %d",
-			      effectf->xmms_format, outputf->xmms_format);
+			      f->xmms_format, outputf->xmms_format);
 			if (outputf->xmms_format < 0)
 				return -1;
 			alsa_convert_func =
 				xmms_convert_get_func(outputf->xmms_format,
-						      effectf->xmms_format);
+						      f->xmms_format);
 			if (alsa_convert_func == NULL)
 				return -1;
 		}
@@ -918,14 +1055,14 @@ static int alsa_setup(struct snd_format *f)
 	}
 
 	snd_pcm_hw_params_set_channels_near(alsa_pcm, hwparams, &outputf->channels);
-	if (outputf->channels != effectf->channels)
+	if (outputf->channels != f->channels)
 	{
 		debug("Converting channels from %d to %d",
-		      effectf->channels, outputf->channels);
+		      f->channels, outputf->channels);
 		alsa_stereo_convert_func =
 			xmms_convert_get_channel_func(outputf->xmms_format,
 						      outputf->channels,
-						      effectf->channels);
+						      f->channels);
 		if (alsa_stereo_convert_func == NULL)
 			return -1;
 	}
@@ -936,10 +1073,10 @@ static int alsa_setup(struct snd_format *f)
 		g_warning("alsa_setup(): No usable samplerate available.");
 		return -1;
 	}
-	if (outputf->rate != effectf->rate)
+	if (outputf->rate != f->rate)
 	{
 		debug("Converting samplerate from %d to %d",
-		      effectf->rate, outputf->rate);
+		      f->rate, outputf->rate);
 		alsa_frequency_convert_func =
 			xmms_convert_get_frequency_func(outputf->xmms_format,
 							outputf->channels);
@@ -947,6 +1084,9 @@ static int alsa_setup(struct snd_format *f)
 			return -1;
 	}
 
+	outputf->sample_bits = snd_pcm_format_physical_width(outputf->format);
+	outputf->bps = (outputf->rate * outputf->sample_bits * outputf->channels) >> 3;
+
 	alsa_buffer_time = alsa_cfg.buffer_time * 1000;
 	if ((err = snd_pcm_hw_params_set_buffer_time_near(alsa_pcm, hwparams,
 							  &alsa_buffer_time, 0)) < 0)
@@ -990,11 +1130,11 @@ static int alsa_setup(struct snd_format *f)
 	}
 
 	alsa_can_pause = snd_pcm_hw_params_can_pause(hwparams);
+	debug("can pause: %d", alsa_can_pause);
 
 	snd_pcm_sw_params_alloca(&swparams);
 	snd_pcm_sw_params_current(alsa_pcm, swparams);
 
-	/* This has effect for non-mmap only */
 	if ((err = snd_pcm_sw_params_set_start_threshold(alsa_pcm,
 			swparams, alsa_buffer_size - alsa_period_size) < 0))
 		g_warning("alsa_setup(): setting start "
@@ -1011,27 +1151,31 @@ static int alsa_setup(struct snd_format *f)
 		snd_pcm_dump(alsa_pcm, logs);
 	}
 
-	bits_per_sample = snd_pcm_format_physical_width(outputf->format);
-	alsa_bps = (outputf->rate * bits_per_sample * outputf->channels) >> 3;
-
-	if (mmap)
+	hw_buffer_size = snd_pcm_frames_to_bytes(alsa_pcm, alsa_buffer_size);
+	hw_period_size = snd_pcm_frames_to_bytes(alsa_pcm, alsa_period_size);
+	if (inputf->bps != outputf->bps)
 	{
-		int chn;
-		buffer = g_malloc(alsa_period_size * bits_per_sample / 8 * outputf->channels);
-		areas = g_malloc0(outputf->channels * sizeof(snd_pcm_channel_area_t));
-		
-		for (chn = 0; chn < outputf->channels; chn++)
-		{
-			areas[chn].addr = buffer;
-			areas[chn].first = chn * bits_per_sample;
-			areas[chn].step = outputf->channels * bits_per_sample;
-		}
+		int align = (inputf->sample_bits * inputf->channels) / 8;
+		hw_buffer_size_in = ((guint64)hw_buffer_size * inputf->bps +
+				     outputf->bps/2) / outputf->bps;
+		hw_period_size_in = ((guint64)hw_period_size * inputf->bps +
+				     outputf->bps/2) / outputf->bps;
+		hw_buffer_size_in -= hw_buffer_size_in % align;
+		hw_period_size_in -= hw_period_size_in % align;
+	}
+	else
+	{
+		hw_buffer_size_in = hw_buffer_size;
+		hw_period_size_in = hw_period_size;
 	}
 
 	debug("Device setup: buffer time: %i, size: %i.", alsa_buffer_time,
-	      snd_pcm_frames_to_bytes(alsa_pcm, alsa_buffer_size));
+	      hw_buffer_size);
+	debug("Device setup: period time: %i, size: %i.", alsa_period_time,
+	      hw_period_size);
 	debug("bits per sample: %i; frame size: %i; Bps: %i",
-	      bits_per_sample, snd_pcm_frames_to_bytes(alsa_pcm, 1), alsa_bps);
+	      snd_pcm_format_physical_width(outputf->format),
+	      snd_pcm_frames_to_bytes(alsa_pcm, 1), outputf->bps);
 
 	return 0;
 }
