@@ -3,6 +3,9 @@
  *                           Thomas Nilsson and 4Front Technologies
  *  Copyright (C) 1999-2004  Haavard Kvaalen
  *
+ *  File name suffix option added by Heikki Orsila 2003
+ *  <heikki.orsila@iki.fi> (no copyrights claimed)
+ * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -31,6 +34,14 @@
 #include "libxmms/configfile.h"
 #include "libxmms/util.h"
 
+
+struct format_info { 
+	AFormat format;
+	int frequency;
+	int channels;
+};
+struct format_info input;
+
 struct wavhead
 {
 	guint32 main_chunk;
@@ -53,6 +64,9 @@ static GtkWidget *path_hbox, *path_label, *path_entry, *path_browse, *path_dirbr
 static GtkWidget *configure_separator;
 static GtkWidget *configure_bbox, *configure_ok, *configure_cancel;
 
+static GtkWidget *use_suffix_toggle = NULL;
+static gboolean use_suffix = FALSE;
+
 static gchar *file_path = NULL;
 static FILE *output_file = NULL;
 static struct wavhead header;
@@ -107,6 +121,7 @@ static void disk_init(void)
 	if (cfgfile)
 	{
 		xmms_cfg_read_string(cfgfile, "disk_writer", "file_path", &file_path);
+		xmms_cfg_read_boolean(cfgfile, "disk_writer", "use_suffix", &use_suffix);
 		xmms_cfg_free(cfgfile);
 	}
 
@@ -127,14 +142,17 @@ static gint disk_open(AFormat fmt, gint rate, gint nch)
 		xmms_show_message(_("Error"),
 				  _("You cannot use the Disk Writer plugin\n"
 				    "when you're running in realtime mode."),
-				  _("Ok"), FALSE, NULL, NULL);
+				  _("OK"), FALSE, NULL, NULL);
 		return 0;
 	}
 
 	pos = xmms_remote_get_playlist_pos(ctrlsocket_get_session_id());
 	title = xmms_remote_get_playlist_file(ctrlsocket_get_session_id(), pos);
-	if (title != NULL && (temp = strrchr(title, '.')) != NULL)
-		*temp = '\0';
+	if (!use_suffix) {
+		if (title != NULL && (temp = strrchr(title, '.')) != NULL) {
+			*temp = '\0';
+		}
+	}
 	if (title == NULL || strlen(g_basename(title)) == 0)
 	{
 		g_free(title);
@@ -177,6 +195,10 @@ static gint disk_open(AFormat fmt, gint rate, gint nch)
 	header.data_length = GUINT32_TO_LE(0);
 	fwrite(&header, sizeof (struct wavhead), 1, output_file);
 
+	input.format = fmt;
+	input.frequency = rate;
+	input.channels = nch;
+
 	return 1;
 }
 
@@ -234,6 +256,26 @@ static void convert_buffer(gpointer buffer, gint length)
 
 static void disk_write(void *ptr, gint length)
 {
+	AFormat new_format;
+	int new_frequency, new_channels;
+	EffectPlugin *ep;
+
+	new_format = input.format;
+	new_frequency = input.frequency;
+	new_channels = input.channels;
+
+	ep = get_current_effect_plugin();
+	if ( effects_enabled() && ep && ep->query_format ) { 
+		ep->query_format(&new_format,&new_frequency,&new_channels);
+	}
+
+	if ( effects_enabled() && ep && ep->mod_samples ) { 
+		length = ep->mod_samples(&ptr,length,
+					input.format,
+					input.frequency,
+					input.channels );
+	}
+
 	if (afmt == FMT_S8 || afmt == FMT_S16_BE ||
 	    afmt == FMT_U16_LE || afmt == FMT_U16_BE || afmt == FMT_U16_NE)
 		convert_buffer(ptr, length);
@@ -314,11 +356,15 @@ static void configure_ok_cb(gpointer data)
 		g_free(file_path);
 	file_path = g_strdup(gtk_entry_get_text(GTK_ENTRY(path_entry)));
 
+	use_suffix =
+		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_suffix_toggle));
+
 	cfgfile = xmms_cfg_open_default_file();
 	if (!cfgfile)
 		cfgfile = xmms_cfg_new();
 
 	xmms_cfg_write_string(cfgfile, "disk_writer", "file_path", file_path);
+	xmms_cfg_write_boolean(cfgfile, "disk_writer", "use_suffix", use_suffix);
 	xmms_cfg_write_default_file(cfgfile);
 	xmms_cfg_free(cfgfile);
 
@@ -335,6 +381,8 @@ static void configure_destroy(void)
 
 static void disk_configure(void)
 {
+	GtkTooltips *use_suffix_tooltips;
+
 	if (!configure_win)
 	{
 		configure_win = gtk_window_new(GTK_WINDOW_DIALOG);
@@ -370,6 +418,14 @@ static void disk_configure(void)
 
 		gtk_widget_show(path_hbox);
 
+		use_suffix_toggle = gtk_check_button_new_with_label(_("Don't strip file name extension"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_suffix_toggle), use_suffix);
+		gtk_box_pack_start(GTK_BOX(configure_vbox), use_suffix_toggle, FALSE, FALSE, 0);
+		use_suffix_tooltips = gtk_tooltips_new();
+		gtk_tooltips_set_tip(use_suffix_tooltips, use_suffix_toggle, "If enabled, the extension from the original filename will not be stripped before adding the .wav extension to the end.", NULL);
+		gtk_tooltips_enable(use_suffix_tooltips);
+		gtk_widget_show(use_suffix_toggle);
+
 		configure_separator = gtk_hseparator_new();
 		gtk_box_pack_start(GTK_BOX(configure_vbox), configure_separator, FALSE, FALSE, 0);
 		gtk_widget_show(configure_separator);
@@ -379,7 +435,7 @@ static void disk_configure(void)
 		gtk_button_box_set_spacing(GTK_BUTTON_BOX(configure_bbox), 5);
 		gtk_box_pack_start(GTK_BOX(configure_vbox), configure_bbox, FALSE, FALSE, 0);
 
-		configure_ok = gtk_button_new_with_label(_("Ok"));
+		configure_ok = gtk_button_new_with_label(_("OK"));
 		gtk_signal_connect(GTK_OBJECT(configure_ok), "clicked", GTK_SIGNAL_FUNC(configure_ok_cb), NULL);
 		GTK_WIDGET_SET_FLAGS(configure_ok, GTK_CAN_DEFAULT);
 		gtk_box_pack_start(GTK_BOX(configure_bbox), configure_ok, TRUE, TRUE, 0);
