@@ -2,7 +2,7 @@
  *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
  *  Copyright (C) 1999       Galex Yen
- *  Copyright (C) 1999-2003  Haavard Kvaalen
+ *  Copyright (C) 1999-2004  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include "config.h"
 #include "esdout.h"
+#include "libxmms/configfile.h"
 
 #include <sys/ioctl.h>
 #ifdef HAVE_OSS
@@ -37,6 +38,11 @@ static void esdout_set_oss_volume(int l, int r);
 static int player = -1;
 static int lp = 100, rp = 100;
 
+void esdout_reset_playerid(void)
+{
+	player = -1;
+}
+
 /*
  * Find the stream id, and set stream volume to 'persistent' value.
  */
@@ -62,35 +68,35 @@ void esdout_fetch_volume(int *l, int *r)
 	esd_info_t *all_info = NULL;
 	esd_player_info_t *info;
 
-	fd = esd_open_sound(esd_cfg.hostname);
-	all_info = esd_get_all_info(fd);
-		
-	/* scan linked list for our playername */
-	for (info = all_info->player_list; info != NULL; info = info->next)
-		if (!strcmp(esd_cfg.playername, info->name))
-			break;
-
-	if (info)
+	if ((fd = esd_open_sound(esd_cfg.hostname)) != -1 &&
+	    (all_info = esd_get_all_info(fd)) != NULL)
 	{
-		player = info->source_id;
-		if (l && r)
-		{
-			/*
-			 * Sometimes we call with NULL
-			 * args to fetch the player num
-			 */
-			*l = (info->left_vol_scale * 100) / 256;
-			*r = (info->right_vol_scale * 100)/ 256;
-		}
-	}
-	else
-		g_warning("xmms: Couldn't find our player "
-			  "(was looking for %s) at the server",
-			  esd_cfg.playername);
+		/* scan linked list for our playername */
+		for (info = all_info->player_list; info != NULL; info = info->next)
+			if (!strcmp(esd_cfg.playername, info->name))
+				break;
 
-	if (all_info)
+		if (info)
+		{
+			player = info->source_id;
+			if (l && r)
+			{
+				/*
+				 * Sometimes we call with NULL
+				 * args to fetch the player num
+				 */
+				*l = (info->left_vol_scale * 100) / 256;
+				*r = (info->right_vol_scale * 100) / 256;
+			}
+		}
+		else
+			g_warning("xmms: Couldn't find our player "
+				  "(was looking for %s) at the server",
+				  esd_cfg.playername);
 		esd_free_all_info(all_info);
-	esd_close(fd);
+	}
+	if (fd != -1)
+		esd_close(fd);
 }
 
 void esdout_get_volume(int *l, int *r)
@@ -116,6 +122,22 @@ void esdout_get_volume(int *l, int *r)
 	}
 }
 
+static void esdout_store_vol(int l, int r)
+{
+	ConfigFile *cfgfile = xmms_cfg_open_default_file();
+
+	xmms_cfg_write_int(cfgfile, "ESD", "volume_l", l);
+	xmms_cfg_write_int(cfgfile, "ESD", "volume_r", r);
+	xmms_cfg_write_default_file(cfgfile);
+	xmms_cfg_free(cfgfile);
+}
+
+void esdout_mixer_init_vol(int l, int r)
+{
+	lp = CLAMP(l, 0, 100);
+	rp = CLAMP(r, 0, 100);
+}
+
 void esdout_set_volume(int l, int r)
 {
 	lp = l;
@@ -134,6 +156,7 @@ void esdout_set_volume(int l, int r)
 					   (l * 256) / 100, (r * 256) / 100);
 			esd_close(fd);
 		}
+		esdout_store_vol(l, r);
 	}
 }
 
