@@ -759,7 +759,7 @@ gint abuffer_open(AFormat fmt, gint rate, gint nch)
 	/* Check the card's capabilities */
 	ctl_dev = g_strconcat(get_audiodev(), "ctl", NULL);
 	/* FIXME: Check if open fails */
-	audiofd = open(ctl_dev, O_RDONLY);
+	audiofd = open(ctl_dev, O_WRONLY);
 	g_free(ctl_dev);
 	ioctl(audiofd, AUDIO_GETDEV, &aud_dev);
 	close(audiofd);
@@ -812,6 +812,19 @@ gint abuffer_open(AFormat fmt, gint rate, gint nch)
 		/* 8 or 16 bit sound */
 		stream_attrs.output_precision = stream_attrs.precision;
 	}
+	else if (!strcmp(aud_dev.name, "SUNW,audio810"))
+	{
+		/* AMD8111 AC'97 audio (SunOS 5.10) */
+		stream_attrs.output_precision = stream_attrs.precision;
+		stream_attrs.is_signed = TRUE;
+		sun_cfg.channel_flags = 0;
+	}       
+	else if (!strcmp(aud_dev.name, "SUNW,oss"))
+	{
+		/* OSS emulating /dev/audio */
+		stream_attrs.output_precision=16;
+		stream_attrs.is_signed = TRUE;
+	}
 	else
 	{
 		g_warning("solaris output: Unknown sound card type: %s", aud_dev.name);
