@@ -19,14 +19,13 @@
  */
 #include "xmms.h"
 
-typedef struct tagRGBQUAD
+struct rgb_quad
 {
 	guchar rgbBlue;
 	guchar rgbGreen;
 	guchar rgbRed;
 	guchar rgbReserved;
-}
-RGBQuad;
+};
 
 struct bitfieldmask {
 	guint32 r, g, b;
@@ -93,11 +92,11 @@ static void get_shift(guint32 v, int *r, int *l)
 		*l = 8 - w;
 }
 
-static void read_1b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[]);
-static void read_4b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[]);
-static void read_4b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[]);
-static void read_8b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[]);
-static void read_8b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[]);
+static void read_1b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette);
+static void read_4b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette);
+static void read_4b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette);
+static void read_8b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette);
+static void read_8b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette);
 static void read_16b_rgb(guint8 *input, int input_size, guint8 *output,
 			 guint32 w, guint32 h, struct bitfieldmask *mask);
 static void read_24b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h);
@@ -114,9 +113,9 @@ GdkPixmap *read_bmp(gchar * filename)
 	guint8 *data, *buffer;
 	struct stat statbuf;
 
-	RGBQuad rgb_quads[256];
+	struct rgb_quad rgb_quads[256];
 	struct bitfieldmask mask = {0};
-	GdkPixmap *ret;
+	GdkPixmap *ret = NULL;
 
 	if (stat(filename, &statbuf) == -1)
 		return NULL;
@@ -127,29 +126,27 @@ GdkPixmap *read_bmp(gchar * filename)
 		return NULL;
 
 	if (fread(type, 1, 2, file) != 2)
-	{
-		fclose(file);
-		return NULL;
-	}
+		goto failure;
 	if (strncmp(type, "BM", 2))
 	{
 		g_warning("read_bmp(): Error in BMP file: wrong type");
-		fclose(file);
-		return NULL;
+		goto failure;
 	}
 	fseek(file, 8, SEEK_CUR);
-	read_le_long(file, &offset);
-	read_le_long(file, &headSize);
+	if (!read_le_long(file, &offset) ||
+	    !read_le_long(file, &headSize))
+		goto failure;
 	if (headSize == 12) /* BITMAPCOREINFO */
 	{
-		guint16 tmp;
+		guint16 tmpw, tmph, dummy;
 		
-		read_le_short(file, &tmp);
-		w = tmp;
-		read_le_short(file, &tmp);
-		h = tmp;
-		read_le_short(file, &tmp); /* planes */
-		read_le_short(file, &bitcount);
+		if (!read_le_short(file, &tmpw) ||
+		    !read_le_short(file, &tmph) ||
+		    !read_le_short(file, &dummy) ||
+		    !read_le_short(file, &bitcount))
+			goto failure;
+		w = tmpw;
+		h = tmph;
 		imgsize = size - offset;
 		comp = BI_RGB;
 	}
@@ -157,29 +154,31 @@ GdkPixmap *read_bmp(gchar * filename)
 	{
 		guint16 tmp;
 		
-		read_le_long(file, &w);
-		read_le_long(file, &h);
-		read_le_short(file, &tmp); /* planes */
-		read_le_short(file, &bitcount);
-		read_le_long(file, &comp);
-		read_le_long(file, &imgsize);
+		if (!read_le_long(file, &w) ||
+		    !read_le_long(file, &h) ||
+		    !read_le_short(file, &tmp) ||
+		    !read_le_short(file, &bitcount) ||
+		    !read_le_long(file, &comp) ||
+		    !read_le_long(file, &imgsize))
+			goto failure;
 		imgsize = size - offset;
 
 		fseek(file, 16, SEEK_CUR);
 	}
 	else
 	{
-		g_warning("read_bmp(): Error in BMP file: Unknown header size");
-		fclose(file);
-		return NULL;
+		g_warning("read_bmp(): Error in BMP file: Invalid header size: %d (%s)",
+			  headSize, filename);
+		goto failure;
 	}
 	if ((bitcount == 16 || bitcount == 32) && comp == BI_BITFIELDS)
 	{
 		if (offset - headSize - 14 >= 12)
 		{
-			read_le_long(file, &mask.r);
-			read_le_long(file, &mask.g);
-			read_le_long(file, &mask.b);
+			if (!read_le_long(file, &mask.r) ||
+			    !read_le_long(file, &mask.g) ||
+			    !read_le_long(file, &mask.b))
+				goto failure;
 		}
 	}
 	else if (bitcount != 24 && bitcount != 16 && bitcount != 32)
@@ -191,18 +190,19 @@ GdkPixmap *read_bmp(gchar * filename)
 		{
 			ncols = MIN(ncols / 3, 256);
 			for (i = 0; i < ncols; i++)
-				fread(&rgb_quads[i], 3, 1, file);
+				if (fread(&rgb_quads[i], 3, 1, file) != 1)
+					goto failure;
 		}
 		else
 		{
 			ncols = MIN(ncols / 4, 256);
-			fread(rgb_quads, 4, ncols, file);
+			if (fread(rgb_quads, 4, ncols, file) != ncols)
+				goto failure;
 		}
 	}
 	fseek(file, offset, SEEK_SET);
 	buffer = g_malloc(imgsize);
 	fread(buffer, imgsize, 1, file);
-	fclose(file);
 	data = g_malloc0((w * 3 * h) + 3);	/* +3 is just for safety */
 
 	if (bitcount == 1)
@@ -244,11 +244,13 @@ GdkPixmap *read_bmp(gchar * filename)
 	g_free(data);
 	g_free(buffer);
 
+ failure:
+	fclose(file);
 	return ret;
 }
 
 
-static void read_1b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[])
+static void read_1b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette)
 {
 	guint8 *input_end = input + input_size;
 	guint8 *output_ptr = output + ((h - 1) * w * 3);
@@ -272,7 +274,7 @@ static void read_1b_rgb(guint8 *input, gint input_size, guint8 *output, guint32
 	}
 }
 
-static void read_4b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[])
+static void read_4b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette)
 {
 	gboolean at_end = 0;
 	gint j;
@@ -375,7 +377,7 @@ static void read_4b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint
 	}
 }
 
-static void read_4b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[])
+static void read_4b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette)
 {
 	guint8 *input_end = input + input_size;
 	guint8 *output_ptr = output + ((h - 1) * w * 3);
@@ -405,7 +407,7 @@ static void read_4b_rgb(guint8 *input, gint input_size, guint8 *output, guint32
 }
 
 
-static void read_8b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[])
+static void read_8b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette)
 {
 	gboolean at_end = 0;
 	gint j;
@@ -499,7 +501,7 @@ static void read_8b_rle(guint8 *input, guint32 compr_size, guint8 *output, guint
 	}
 }
 
-static void read_8b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, RGBQuad palette[])
+static void read_8b_rgb(guint8 *input, gint input_size, guint8 *output, guint32 w, guint32 h, struct rgb_quad *palette)
 {
 	guint8 *input_end = input + input_size;
 	guint8 *output_ptr = output + ((h - 1) * w * 3);
