@@ -425,6 +425,10 @@ void *ctrlsocket_func(void *arg)
 				ctrl_write_gboolean(pkt->fd, cfg.shuffle);
 				ctrl_ack_packet(pkt);
 				break;
+			case CMD_IS_ADVANCE:
+				ctrl_write_gboolean(pkt->fd, !cfg.no_playlist_advance);
+				ctrl_ack_packet(pkt);
+				break;
 			case CMD_GET_EQ:
 				fval[0] = equalizerwin_get_preamp();
 				for(i = 0; i < 10; i++)
@@ -441,6 +445,20 @@ void *ctrlsocket_func(void *arg)
 				ctrl_write_gfloat(pkt->fd, equalizerwin_get_band(i));
 				ctrl_ack_packet(pkt);
 				break;
+			case CMD_GET_PLAYQUEUE_LENGTH:
+				ctrl_write_gint(pkt->fd, get_playlist_queue_length());
+				ctrl_ack_packet(pkt);
+				break;
+			case CMD_GET_PLAYQUEUE_POS_FROM_PLAYLIST_POS:
+				i = *((guint32 *) pkt->data);
+				ctrl_write_gint(pkt->fd, playlist_get_playqueue_position_from_playlist_position(i));
+				ctrl_ack_packet(pkt);
+				break;
+			case CMD_GET_PLAYLIST_POS_FROM_PLAYQUEUE_POS:
+				i = *((guint32 *) pkt->data);
+				ctrl_write_gint(pkt->fd, playlist_get_playlist_position_from_playqueue_position(i));
+				ctrl_ack_packet(pkt);
+				break;
 			default:
 				pthread_mutex_lock(&packet_list_mutex);
 				packet_list = g_list_append(packet_list, pkt);
@@ -549,6 +567,10 @@ void check_ctrlsocket(void)
 				mainwin_shuffle_pushed(!cfg.shuffle);
 				tbutton_set_toggled(mainwin_shuffle, GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu, "/Shuffle"))->active);
 				break;
+			case CMD_TOGGLE_ADVANCE:
+				mainwin_advance_pushed(!cfg.no_playlist_advance);
+				/* there is no real button, so no need to call tbutton_set_toggled() */
+				break;
 			case CMD_MAIN_WIN_TOGGLE:
 				tbool = *((gboolean *) data);
 				mainwin_show(!!tbool);
@@ -584,6 +606,19 @@ void check_ctrlsocket(void)
 				pthread_mutex_unlock(&packet_list_mutex);
 				mainwin_quit_cb();
 				break;
+			case CMD_PLAYQUEUE_ADD:
+				num = *((guint32 *) data);
+				if (num < get_playlist_length())
+					playlist_queue_position(num);
+				break;
+			case CMD_PLAYQUEUE_REMOVE:
+				num = *((guint32 *) data);
+				if (num < get_playlist_length())
+					playlist_queue_remove(num);
+				break;
+			case CMD_PLAYQUEUE_CLEAR:
+				playlist_clear_queue();
+				break;
 			default:
 				g_log(NULL, G_LOG_LEVEL_MESSAGE, "Unknown socket command received");
 				break;
