@@ -1,5 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1998-2006  Peter Alm, Mikael Alm, Olle Hallnas,
+ *                           Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999-2006  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,39 +20,48 @@
 #include "xmms.h"
 
 struct EffectPluginData *ep_data;
+static pthread_mutex_t emutex = PTHREAD_MUTEX_INITIALIZER;
 
-static int effect_do_mod_samples(gpointer *data, gint length, AFormat fmt, gint srate, gint nch)
+static int effect_do_mod_samples(gpointer *data, int length, AFormat fmt, int srate, int nch)
 {
-	GList *l = ep_data->enabled_list;
+	GList *l;
 
-	while(l)
+	pthread_mutex_lock(&emutex);
+	l = ep_data->enabled_list;
+
+	while (l)
 	{
-		if(l->data)
+		if (l->data)
 		{
 			EffectPlugin *ep = l->data;
-			if(ep->mod_samples)
+			if (ep->mod_samples)
 				length = ep->mod_samples(data, length, fmt, srate, nch);
 		}
 		l = g_list_next(l);
 	}
+	pthread_mutex_unlock(&emutex);
 	
 	return length;
 }
 
-static void effect_do_query_format(AFormat *fmt, gint *rate, gint *nch)
+static void effect_do_query_format(AFormat *fmt, int *rate, int *nch)
 {
-	GList *l = ep_data->enabled_list;
+	GList *l;
 
-	while(l)
+	pthread_mutex_lock(&emutex);
+	l = ep_data->enabled_list;
+
+	while (l)
 	{
-		if(l->data)
+		if (l->data)
 		{
 			EffectPlugin *ep = l->data;
-			if(ep->query_format)
+			if (ep->query_format)
 				ep->query_format(fmt, rate, nch);
 		}
 		l = g_list_next(l);
 	}	
+	pthread_mutex_unlock(&emutex);
 }
 
 static EffectPlugin pseudo_effect_plugin =
@@ -81,18 +92,12 @@ int effects_enabled(void)
 	return TRUE;
 }
 
-GList *get_effect_enabled_list(void)
-{
-	return ep_data->enabled_list;
-}
-
 void effect_about(int i)
 {
-	EffectPlugin *effect;
 	GList *node = g_list_nth(ep_data->effect_list, i);
 	if (node)
 	{
-		effect = node->data;
+		EffectPlugin *effect = node->data;
 		if (effect && effect->about)
 			effect->about();
 	}
@@ -101,10 +106,9 @@ void effect_about(int i)
 void effect_configure(int i)
 {
 	GList *node = g_list_nth(ep_data->effect_list, i);
-	EffectPlugin *effect;
 	if (node)
 	{
-		effect = node->data;
+		EffectPlugin *effect = node->data;
 		if (effect && effect->configure)
 			effect->configure();
 	}
@@ -116,22 +120,24 @@ void enable_effect_plugin(int i, gboolean enable)
 	GList *node = g_list_nth(ep_data->effect_list, i);
 	EffectPlugin *ep;
 	
-	if(!node || !(node->data))
+	if (!node || !(node->data))
 		return;
 	ep = node->data;
 	
-	if(enable && !g_list_find(ep_data->enabled_list, ep))
+	pthread_mutex_lock(&emutex);
+	if (enable && !g_list_find(ep_data->enabled_list, ep))
 	{
 		ep_data->enabled_list = g_list_append(ep_data->enabled_list, ep);
 		if (ep->init)
 			ep->init();
 	}
-	else if(!enable && g_list_find(ep_data->enabled_list, ep))
+	else if (!enable && g_list_find(ep_data->enabled_list, ep))
 	{
 		ep_data->enabled_list = g_list_remove(ep_data->enabled_list, ep);
 		if (ep->cleanup)
 			ep->cleanup();
 	}
+	pthread_mutex_unlock(&emutex);
 }
 
 GList *get_effect_list(void)
@@ -141,50 +147,52 @@ GList *get_effect_list(void)
 
 gboolean effect_enabled(int i)
 {
-	return (g_list_find(ep_data->enabled_list, (EffectPlugin *) g_list_nth(ep_data->effect_list, i)->data) ? TRUE : FALSE);
+	GList *pl = g_list_nth(ep_data->effect_list, i);
+	if (g_list_find(ep_data->enabled_list, pl->data))
+		return TRUE;
+	return FALSE;
 }
 
 gchar *effect_stringify_enabled_list(void)
 {
-	gchar *enalist = NULL, *temp, *temp2;
+	char *enalist = NULL, *temp;
 	GList *node = ep_data->enabled_list;
 
 	if (g_list_length(node))
 	{
-		enalist = g_strdup(g_basename(((EffectPlugin *) node->data)->filename));
+		EffectPlugin *ep = node->data;
+		enalist = g_strdup(g_basename(ep->filename));
 		node = node->next;
 		while (node)
 		{
 			temp = enalist;
-			temp2 = g_strdup(g_basename(((EffectPlugin *) node->data)->filename));
-			enalist = g_strconcat(temp, ",", temp2, NULL);
+			ep = node->data;
+			enalist = g_strconcat(temp, ",", g_basename(ep->filename), NULL);
 			g_free(temp);
-			g_free(temp2);
 			node = node->next;
 		}
 	}
 	return enalist;
 }
 
-void effect_enable_from_stringified_list(gchar * list)
+void effect_enable_from_stringified_list(char * list)
 {
-	gchar **plugins, *base;
+	char **plugins, *base;
 	GList *node;
-	gint i;
-	EffectPlugin *ep;
+	int i;
 
-	if (!list || !strcmp(list, ""))
+	if (!list || list[0] == '\0')
 		return;
 	plugins = g_strsplit(list, ",", 0);
-	for(i = 0; plugins[i]; i++)
+	for (i = 0; plugins[i]; i++)
 	{
 		node = ep_data->effect_list;
 		while (node)
 		{
-			base = g_basename(((EffectPlugin *) node->data)->filename);
+			EffectPlugin *ep = node->data;
+			base = g_basename(ep->filename);
 			if (!strcmp(plugins[i], base))
 			{
-				ep = node->data;
 				ep_data->enabled_list = g_list_append(ep_data->enabled_list, ep);
 				if(ep->init)
 					ep->init();
