@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2004  Haavard Kvaalen
+ *  Copyright (C) 1999-2006  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@ static void playlist_load_ins_file(char *filename, char *playlist_name, long pos
 static int __get_playlist_length(void);
 static void playlist_generate_shuffle_list(void);
 static void __playlist_generate_shuffle_list(void);
+static GList *playlist_shuffle_list(GList *list);
 
 struct devino {
 	dev_t dev;
@@ -81,7 +82,7 @@ void playlist_clear(void)
 	if (get_input_playing())
 		input_stop();
 
-	pthread_mutex_lock(&playlist_mutex);
+	PL_LOCK();
 	if (playlist)
 	{
 		node = playlist;
@@ -100,7 +101,13 @@ void playlist_clear(void)
 		playlist_position = NULL;
 	}
 
-	pthread_mutex_unlock(&playlist_mutex);
+	if (queued_list)
+	{
+		g_list_free(queued_list);
+		queued_list = NULL;
+	}
+
+	PL_UNLOCK();
 	playlist_generate_shuffle_list();
 	playlistwin_update_list();
 }
@@ -108,16 +115,27 @@ void playlist_clear(void)
 void playlist_delete_node(GList *node, gboolean *set_info_text, gboolean *restart_playing)
 {
 	/* Caller should hold playlist mutex */
-	PlaylistEntry *entry;
+	PlaylistEntry *entry = node->data;
 	GList *playing_song = NULL;
 	GList *queued_song = NULL;
+
 	/*
 	 * We call g_list_find manually here because
 	 * we don't want an item in the shuffle_list
 	 */
 	if (playlist_position)
 		playing_song = g_list_find(playlist, playlist_position);
-	entry = node->data;
+
+	/*
+	 * Remove the song from the queue first if it is there, so
+	 * that that we avoid trying to play it again
+	 */
+	if ((queued_song = g_list_find(queued_list, entry)) != NULL)
+	{
+		queued_list = g_list_remove_link(queued_list, queued_song);
+		g_list_free_1(queued_song);
+	}
+
 	if (playing_song == node)
 	{
 		*set_info_text = TRUE;
@@ -129,7 +147,9 @@ void playlist_delete_node(GList *node, gboolean *set_info_text, gboolean *restar
 			*restart_playing = TRUE;
 		}
 		playing_song = find_playlist_position_list();
-		if (g_list_next(playing_song))
+		if (queued_list)
+			play_queued();
+		else if (g_list_next(playing_song))
 			playlist_position = playing_song->next->data;
 		else if (g_list_previous(playing_song))
 			playlist_position = playing_song->prev->data;
@@ -144,12 +164,6 @@ void playlist_delete_node(GList *node, gboolean *set_info_text, gboolean *restar
 		 g_list_position(playlist, node))
 		*set_info_text = TRUE;
 
-	if ((queued_song = g_list_find(queued_list, entry)) != NULL)
-	{
-		queued_list = g_list_remove_link(queued_list, queued_song);
-		g_list_free_1(queued_song);
-	}
-
 	if (entry->filename)
 		g_free(entry->filename);
 	if (entry->title)
@@ -193,7 +207,6 @@ void playlist_delete_index(glong index)
 	}
 	else if (set_info_text)
 		mainwin_set_info_text();
-
 }
 
 void playlist_delete_filenames(GList *filenames)
@@ -236,7 +249,7 @@ void playlist_delete(gboolean crop)
 	GList *node, *next;
 	PlaylistEntry *entry;
 
-	pthread_mutex_lock(&playlist_mutex);
+	PL_LOCK();
 
 	node = playlist;
 
@@ -248,11 +261,9 @@ void playlist_delete(gboolean crop)
 			playlist_delete_node(node, &set_info_text, &restart_playing);
 		node = next;
 	}
-	pthread_mutex_unlock(&playlist_mutex);
+	PL_UNLOCK();
 	
 	playlistwin_update_list();
-	if (set_info_text)
-		mainwin_set_info_text();
 	if (restart_playing)
 	{
 		if (playlist_position)
@@ -260,6 +271,8 @@ void playlist_delete(gboolean crop)
 		else
 			mainwin_clear_song_info();
 	}
+	else if (set_info_text)
+		mainwin_set_info_text();	
 }
 
 static void __playlist_ins_with_info(char *filename, long pos, char* title, int len)
@@ -272,9 +285,9 @@ static void __playlist_ins_with_info(char *filename, long pos, char* title, int
 		entry->title = g_strdup(title);
 	entry->length = len;
 
-	pthread_mutex_lock(&playlist_mutex);
+	PL_LOCK();
 	playlist = g_list_insert(playlist, entry, pos);
-	pthread_mutex_unlock(&playlist_mutex);		
+	PL_UNLOCK();
 
 	playlist_get_info_scan_active = TRUE;
 }
@@ -551,7 +564,7 @@ void playlist_play(void)
 		/* If there are entries */
 		if (g_list_length(skinlist)) {
 			/* Get a random value to select the skin to use */
-			int randval = random() % (g_list_length(skinlist) + 1);
+			int randval = (gint)(random() / (RAND_MAX + 1.0) * (g_list_length(skinlist) + 1));
 			/* If the random value is 0, use the default skin */
 			/* Otherwise subtract 1 from the random value and */
 			/* select the skin */
@@ -658,7 +671,7 @@ void playlist_next(void)
 
 	plist_pos_list = find_playlist_position_list();
 
-	if (!cfg.repeat && !g_list_next(plist_pos_list))
+	if (!cfg.repeat && !g_list_next(plist_pos_list) && !queued_list)
 	{
 		PL_UNLOCK();
 		return;
@@ -769,11 +782,21 @@ static void playlist_queue_toggle(void *data)
 		queued_list = g_list_append(queued_list, data);
 }
 
+gint get_playlist_queue_length(void)
+{
+	gint qlength;
+
+	PL_LOCK();
+	qlength = g_list_length(queued_list);
+	PL_UNLOCK();
+
+	return qlength;
+}
 
 void playlist_queue_selected(void)
 {
 	GList *node;
- 
+
 	PL_LOCK();
 	for (node = get_playlist(); node; node = node->next)
 	{
@@ -785,50 +808,99 @@ void playlist_queue_selected(void)
 		playlist_queue_toggle(entry);
 	}
 	PL_UNLOCK();
- 
+
 	playlistwin_update_list();
 }
- 
+
 void playlist_queue_position(int pos)
 {
 	void *entry;
- 
+
 	PL_LOCK();
 	entry = g_list_nth_data(playlist, pos);
 	playlist_queue_toggle(entry);
 	PL_UNLOCK();
- 
+
 	playlistwin_update_list();
 }
- 
+
 gboolean playlist_is_position_queued(int pos)
 {
 	PlaylistEntry *entry;
 	GList *temp;
- 
+
 	PL_LOCK();
 	entry = g_list_nth_data(playlist, pos);
 	temp = g_list_find(queued_list, entry);
 	PL_UNLOCK();
- 
+
 	return temp != NULL;
 }
- 
+
+gboolean playlist_is_position_selected(int pos)
+{
+	PlaylistEntry *entry;
+	gboolean selected;
+
+	PL_LOCK();
+	entry = g_list_nth_data(playlist, pos);
+	selected = entry && entry->selected;
+	PL_UNLOCK();
+
+	return selected;
+}
+
 void playlist_clear_queue(void)
 {
 	PL_LOCK();
 	g_list_free(queued_list);
 	queued_list = NULL;
 	PL_UNLOCK();
- 
+
 	playlistwin_update_list();
 }
- 
+
+void playlist_queue_remove(int pos)
+{
+	void *entry;
+
+	PL_LOCK();
+	entry = g_list_nth_data(playlist, pos);
+	queued_list = g_list_remove(queued_list, entry);
+	PL_UNLOCK();
+
+	playlistwin_update_list();
+}
+
+void playlist_queue_move(int oldpos, int newpos)
+{
+	GList *tmp;
+
+	PL_LOCK();
+	tmp = g_list_nth(queued_list, oldpos);
+	queued_list = g_list_remove_link (queued_list, tmp);
+	queued_list = g_list_insert(queued_list, tmp->data, newpos);
+	g_list_free_1 (tmp);
+	PL_UNLOCK();
+	
+	playlistwin_update_list();
+}
+
 int playlist_get_queue_position(PlaylistEntry *entry)
 {
 	return g_list_index(queued_list, entry);
 }
 
+int playlist_get_playqueue_position_from_playlist_position(int pos)
+{
+	return playlist_get_queue_position(g_list_nth_data(playlist, pos));
+}
+
+int playlist_get_playlist_position_from_playqueue_position(int pos)
+{
+	return g_list_index(playlist, g_list_nth_data(queued_list, pos));
+}
+
 void playlist_set_position(int pos)
 {
 	GList *node;
@@ -900,7 +972,7 @@ void playlist_eof_reached(void)
 		if (cfg.shuffle)
 		{
 			playlist_position = NULL;
-  			__playlist_generate_shuffle_list();
+			__playlist_generate_shuffle_list();
 		}
 		else
 			playlist_position = playlist->data;
@@ -1060,12 +1132,11 @@ gboolean playlist_load(char * filename)
 	return playlist_load_ins(filename, -1);
 }
 
-static void playlist_load_ins_file(char *filename, char *playlist_name, long pos, char *title, int len)
+static void playlist_load_ins_file(char *filename, char *playlist_name,
+				   long pos, char *title, int len)
 {
 	char *temp, *path;
 
-	filename = g_strstrip(filename);
-
 	if (cfg.use_backslash_as_dir_delimiter)
 	{
 		while ((temp = strchr(filename, '\\')) != NULL)
@@ -1135,7 +1206,6 @@ static guint playlist_load_ins(char * filename, long pos)
 	if (ext && !strcasecmp(ext, ".pls"))
 	{
 		int noe, i;
-		char key[10];
 		
 		line = read_ini_string(filename, "playlist", "NumberOfEntries");
 		if (line == NULL)
@@ -1146,18 +1216,26 @@ static guint playlist_load_ins(char * filename, long pos)
 
 		for (i = 1; i <= noe; i++)
 		{
-			g_snprintf(key, 10, "File%d", i);
+			char key[15];
+			g_snprintf(key, 15, "File%d", i);
 			line = read_ini_string_no_comment(filename,
 							  "playlist", key);
+			g_snprintf(key, 15, "Title%d", i);
+			if (cfg.use_pl_metadata)
+				ext_title = read_ini_string_no_comment(
+						filename, "playlist", key);
+			else
+				ext_title = NULL;
 			if (line != NULL)
 			{
 				playlist_load_ins_file(line, filename, pos,
-						       NULL, -1);
+						       ext_title, -1);
 				entries++;
 				if (pos >= 0)
 					pos++;
 				g_free(line);
 			}
+			g_free(ext_title);
 		}
 		playlist_generate_shuffle_list();
 		playlistwin_update_list();
@@ -1243,6 +1321,12 @@ GList *get_playlist(void)
 	return playlist;
 }
 
+GList *get_queue(void)
+{
+	/* Caller should hold playlist_mutex */
+	return queued_list;
+} 
+
 int __get_playlist_position(void)
 {
 	/* Caller should hold playlist_mutex */
@@ -1438,18 +1522,18 @@ static int playlist_sort_str_by_path_cmpfunc(gconstpointer pa, gconstpointer pb)
 	 */
 	if (posa && posb && (posa - a != posb - b))
 	{
-		int len, ret;
-		if (posa - a > posb - b)
+		int len, ret = 0;
+		if (posa - a > posb - b && a[posb - b] == '/')
 		{
 			len = posb - b;
 			ret = -1;
 		}
-		else
+		else if (posb - b > posa - a && b[posa - a] == '/')
 		{
 			len = posa - a;
 			ret = 1;
 		}
-		if (!strncasecmp(a, b, len))
+		if (ret && !strncasecmp(a, b, len))
 			return ret;
 	}
 	return strcasecmp(a, b);
@@ -1506,7 +1590,7 @@ static GList* playlist_sort_selected(GList *list, GCompareFunc cmpfunc)
 
 	/*
 	 * We take all the selected entries out of the playlist,
-	 * sorts them, and then put them back in again.
+	 * sort them, and then put them back in again.
 	 */
 
 	list1 = g_list_last(list);
@@ -1524,8 +1608,12 @@ static GList* playlist_sort_selected(GList *list, GCompareFunc cmpfunc)
 		}
 		list1 = list2;
 	}
-
-	temp_list = g_list_sort(temp_list, cmpfunc);
+	
+	if (cmpfunc)
+		temp_list = g_list_sort(temp_list, cmpfunc);
+	else
+		temp_list = playlist_shuffle_list(temp_list);
+		
 	list1 = temp_list;
 	list2 = index_list;
 
@@ -1605,14 +1693,14 @@ static GList *playlist_shuffle_list(GList *list)
 	for (node = list, i = 0; i < len; node = g_list_next(node), i++)
 		ptrs[i] = node;
 
-	j = random() % len;
+	j = (int)(random() / (RAND_MAX + 1.0) * len); 
 	list = ptrs[j];
 	ptrs[j]->next = NULL;
 	ptrs[j] = ptrs[0];
 
 	for (i = 1; i < len; i++)
 	{
-		j = random() % (len - i);
+		j = (int)(random() / (RAND_MAX + 1.0) * (len - i));
 		list->prev = ptrs[i + j];
 		ptrs[i + j]->next = list;
 		list = ptrs[i + j];
@@ -1634,6 +1722,13 @@ void playlist_random(void)
 	PL_UNLOCK();
 }
 
+void playlist_randomize_selected(void)
+{
+	PL_LOCK();
+	playlist = playlist_sort_selected(playlist, NULL);
+	PL_UNLOCK();
+}
+
 GList * playlist_get_selected(void)
 {
 	GList *node, *list = NULL;
@@ -1888,7 +1983,7 @@ void playlist_start_get_info_scan(void)
 void playlist_remove_dead_files(void)
 {
 	/* FIXME? Does virtual directories work well? */
-	GList *node, *next_node, *curr_pos = NULL, *temp = NULL;
+	GList *node, *next_node, *temp = NULL;
 	PlaylistEntry *entry;
 	gboolean list_changed = FALSE;
 	
@@ -1905,33 +2000,17 @@ void playlist_remove_dead_files(void)
 		    ((temp = input_scan_dir(entry->filename)) == NULL) &&
 		    access(entry->filename, R_OK))
 		{
-			list_changed = TRUE;
-			if (playlist_position)
-				curr_pos = g_list_find(playlist, playlist_position);
-			if (node == curr_pos)
+			gboolean set_info_text = FALSE, restart_playing = FALSE;
+			if (entry == playlist_position && get_input_playing())
 			{
-				if (get_input_playing())
-				{
-					/* Don't remove the currently
-					   playing song */
-					node = next_node;
-					continue;
-				}
-				if (g_list_next(curr_pos))
-					playlist_position = curr_pos->next->data;
-				else if (g_list_previous(curr_pos))
-					playlist_position = curr_pos->prev->data;
-				else if (node != playlist)
-					playlist_position = playlist->data;
-				else
-					playlist_position = NULL;
+				/* Don't remove the currently playing song */
+				node = next_node;
+				continue;
 			}
 
-			playlist = g_list_remove_link(playlist, node);
-			g_free(entry->title);
-			g_free(entry->filename);
-			g_free(node->data);
-			g_list_free_1(node);
+			playlist_delete_node(node, &set_info_text, &restart_playing);
+			if (set_info_text)
+				list_changed = TRUE;
 		}
 		else if (temp)
 		{
