@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2004  Haavard Kvaalen
+ *  Copyright (C) 1999-2006  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,10 +22,10 @@
 
 typedef struct
 {
-	gchar *filename;
-	gchar *title;
-	gint length;
-	gboolean selected;
+        gchar *filename;
+        gchar *title;
+        gint length;
+        gboolean selected;
 }
 PlaylistEntry;
 
@@ -48,8 +48,14 @@ void playlist_prev(void);
 void playlist_queue_selected(void);
 void playlist_queue_position(gint pos);
 gboolean playlist_is_position_queued(int pos);
+gboolean playlist_is_position_selected(int pos);
+gint get_playlist_queue_length(void);
 void playlist_clear_queue(void);
+void playlist_queue_remove(int pos);
+void playlist_queue_move(int oldpos, int newpos);
 int playlist_get_queue_position(PlaylistEntry *entry);
+int playlist_get_playqueue_position_from_playlist_position(int pos);
+int playlist_get_playlist_position_from_playqueue_position(int pos);
 void playlist_eof_reached(void);
 void playlist_set_position(gint pos);
 gint get_playlist_length(void);
@@ -60,6 +66,7 @@ int playlist_get_current_length(void);
 gboolean playlist_save(char *filename, gboolean is_pls);
 gboolean playlist_load(gchar * filename);
 GList *get_playlist(void);
+GList *get_queue(void);
 void playlist_start_get_info_thread(void);
 void playlist_stop_get_info_thread();
 void playlist_start_get_info_scan(void);
@@ -73,6 +80,7 @@ void playlist_sort_selected_by_path(void);
 void playlist_sort_selected_by_date(void);
 void playlist_reverse(void);
 void playlist_random(void);
+void playlist_randomize_selected(void);
 void playlist_remove_dead_files(void);
 void playlist_fileinfo_current(void);
 void playlist_fileinfo(gint pos);
