@@ -1,5 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas,
+ *                           Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999-2001  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,7 +27,7 @@ typedef struct
 	gboolean ps_is_draging;
 	gint ps_drag_y, ps_prev_y, ps_prev_height;
 	GdkImage *ps_back_image;
-	gchar *ps_old_skin_path;
+	int ps_skin_id;
 }
 PlaylistSlider;
 
