@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2004  Haavard Kvaalen
+ *  Copyright (C) 1999-2006  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -68,7 +68,7 @@ enum
 	SEL_INV, SEL_ZERO, SEL_ALL,
 	MISC_SORT, MISC_FILEINFO, MISC_MISCOPTS, MISC_QUEUE,
 	PLIST_NEW, PLIST_SAVE, PLIST_LOAD,
-	SEL_LOOKUP,
+	SEL_LOOKUP, MISC_QUEUE_MANAGER
 };
 
 static void playlistwin_sort_menu_callback(gpointer cb_data, guint action, GtkWidget * w);
@@ -86,7 +86,8 @@ enum
 	PLAYLISTWIN_SORT_BYPATH, PLAYLISTWIN_SORT_BYDATE,
 	PLAYLISTWIN_SORT_SEL_BYTITLE, PLAYLISTWIN_SORT_SEL_BYFILENAME,
 	PLAYLISTWIN_SORT_SEL_BYPATH, PLAYLISTWIN_SORT_SEL_BYDATE,
-	PLAYLISTWIN_SORT_RANDOMIZE, PLAYLISTWIN_SORT_REVERSE
+	PLAYLISTWIN_SORT_RANDOMIZE, PLAYLISTWIN_SORT_REVERSE,
+	PLAYLISTWIN_SORT_SEL_RANDOMIZE,
 };
 
 GtkItemFactoryEntry playlistwin_sort_menu_entries[] =
@@ -100,15 +101,17 @@ GtkItemFactoryEntry playlistwin_sort_menu_entries[] =
 					PLAYLISTWIN_SORT_BYPATH, "<Item>"},
 	{N_("/Sort List/By Date"), NULL, playlistwin_sort_menu_callback,
 					PLAYLISTWIN_SORT_BYDATE, "<Item>"},
-	{N_("/Sort Selection"), NULL, NULL, 0, "<Branch>"},
-	{N_("/Sort Selection/By Title"), NULL, playlistwin_sort_menu_callback,
+	{N_("/Selection"), NULL, NULL, 0, "<Branch>"},
+	{N_("/Selection/Sort By Title"), NULL, playlistwin_sort_menu_callback,
 					PLAYLISTWIN_SORT_SEL_BYTITLE, "<Item>"},
-	{N_("/Sort Selection/By Filename"), NULL, playlistwin_sort_menu_callback,
+	{N_("/Selection/Sort By Filename"), NULL, playlistwin_sort_menu_callback,
 					PLAYLISTWIN_SORT_SEL_BYFILENAME, "<Item>"},
-	{N_("/Sort Selection/By Path + Filename"), NULL, playlistwin_sort_menu_callback,
+	{N_("/Selection/Sort By Path + Filename"), NULL, playlistwin_sort_menu_callback,
 					PLAYLISTWIN_SORT_SEL_BYPATH, "<Item>"},
-	{N_("/Sort Selection/By Date"), NULL, playlistwin_sort_menu_callback,
+	{N_("/Selection/Sort By Date"), NULL, playlistwin_sort_menu_callback,
 					PLAYLISTWIN_SORT_SEL_BYDATE, "<Item>"},
+	{N_("/Selection/Randomize"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_SEL_RANDOMIZE, "<Item>"},
 	{"/-", NULL, NULL, 0, "<Separator>"},
 	{N_("/Randomize List"), NULL, playlistwin_sort_menu_callback,
 					PLAYLISTWIN_SORT_RANDOMIZE, "<Item>"},
@@ -154,6 +157,7 @@ GtkItemFactoryEntry playlistwin_popup_menu_entries[] =
 {
 	{N_("/View File Info"), "<control>3", playlistwin_popup_menu_callback, MISC_FILEINFO, "<Item>"},
 	{N_("/Queue - Unqueue"), "Q", playlistwin_popup_menu_callback, MISC_QUEUE, "<Item>"},
+	{N_("/Queue manager"), "<alt>Q", playlistwin_popup_menu_callback, MISC_QUEUE_MANAGER, "<Item>"},
 	{N_("/-"), NULL, NULL, 0, "<Separator>"},
 	{N_("/Add"), NULL, NULL, 0, "<Branch>"},
 	{N_("/Add/File"), NULL, playlistwin_popup_menu_callback, ADD_FILE, "<Item>"},
@@ -689,16 +693,19 @@ static void playlistwin_set_sensitive_sortmenu(void)
 	set = playlist_get_num_selected() > 1;
 
 	w = gtk_item_factory_get_widget(playlistwin_sort_menu,
-					"/Sort Selection/By Title");
+					"/Selection/Sort By Title");
 	gtk_widget_set_sensitive(w, set);
 	w = gtk_item_factory_get_widget(playlistwin_sort_menu,
-					"/Sort Selection/By Filename");
+					"/Selection/Sort By Filename");
 	gtk_widget_set_sensitive(w, set);
 	w = gtk_item_factory_get_widget(playlistwin_sort_menu,
-					"/Sort Selection/By Path + Filename");
+					"/Selection/Sort By Path + Filename");
 	gtk_widget_set_sensitive(w, set);
 	w = gtk_item_factory_get_widget(playlistwin_sort_menu,
-					"/Sort Selection/By Date");
+					"/Selection/Sort By Date");
+	gtk_widget_set_sensitive(w, set);
+	w = gtk_item_factory_get_widget(playlistwin_sort_menu,
+					"/Selection/Randomize");
 	gtk_widget_set_sensitive(w, set);
 }
 
@@ -723,7 +730,7 @@ static void playlistwin_save_playlist_error(char* path, GtkWidget *filesel)
 	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), bbox, FALSE, FALSE, 0);
 
-	close = gtk_button_new_with_label(_("Ok"));
+	close = gtk_button_new_with_label(_("OK"));
 	gtk_signal_connect_object(GTK_OBJECT(close), "clicked",
 				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
 				  GTK_OBJECT(dialog));
@@ -829,10 +836,12 @@ static void playlistwin_save_filesel_ok(GtkWidget * w, GtkFileSelection * filese
 		{
 			char *m;
 			GtkWidget *w;
-			m = g_strdup_printf(_("Unknown file type for %s"),
+			m = g_strdup_printf(_("Unknown file type for %s.\n"
+					      "The filename of the playlist should end "
+					      "in either \".m3u\" or \".pls\"."),
 					    filename);
 			w = xmms_show_message(_("Unable to save playlist"), m,
-					      _("Ok"), TRUE, NULL, NULL);
+					      _("OK"), TRUE, NULL, NULL);
 			gtk_window_set_transient_for(GTK_WINDOW(w),
 						     GTK_WINDOW(filesel));
 
@@ -1226,51 +1235,42 @@ static void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpoint
 		if(dock_is_moving(playlistwin))
 			dock_move_release(playlistwin);
 	}
-	else if (event->button == 3 &&
-		 !(inside_widget(event->x, event->y, playlistwin_list) ||
-		   (event->y >= cfg.playlist_height - 29 &&
-		    event->y < cfg.playlist_height - 11 &&
-		    ((event->x >= 12 && event->x < 37) ||
-		     (event->x >= 41 && event->x < 66) ||
-		     (event->x >= 70 && event->x < 95) ||
-		     (event->x >= 99 && event->x < 124) ||
-		     (event->x >= cfg.playlist_width - 46 &&
-		      event->x < cfg.playlist_width - 23)))))
+	else if (event->button == 3)
 	{
-		/*
-		 * Pop up the main menu a few pixels down to avoid
-		 * anything to be selected initially.
-		 */
-		util_item_factory_popup(mainwin_general_menu, event->x_root,
-					event->y_root + 2, 3, event->time);
-		grab = FALSE;
-	}
-	else if (event->button == 3 &&
-		 inside_widget(event->x, event->y, playlistwin_list))
-	{
-		int pos, sensitive;
-		GtkWidget *w;
-		pos = playlist_list_get_playlist_position(playlistwin_list,
-							  event->x, event->y);
-		if (pos != -1)
-			pos++;
-		sensitive = pos != -1;
-		w = gtk_item_factory_get_widget(playlistwin_popup_menu,
-						"/View File Info");
-		gtk_widget_set_sensitive(w, sensitive);
+		if (!inside_widget(event->x, event->y, playlistwin_list))
+			/*
+			 * Pop up the main menu a few pixels down to avoid
+			 * anything to be selected initially.
+			 */
+			util_item_factory_popup(mainwin_general_menu,
+						event->x_root, event->y_root + 2,
+						3, event->time);
+		else
+		{
+			int pos, sensitive;
+			GtkWidget *w;
+			pos = playlist_list_get_playlist_position(playlistwin_list,
+								  event->x, event->y);
+			if (pos != -1)
+				pos++;
+			sensitive = pos != -1;
+			w = gtk_item_factory_get_widget(playlistwin_popup_menu,
+							"/View File Info");
+			gtk_widget_set_sensitive(w, sensitive);
 
-		w = gtk_item_factory_get_widget(playlistwin_popup_menu,
-						"/Queue - Unqueue");
-		gtk_widget_set_sensitive(w, sensitive);
+			w = gtk_item_factory_get_widget(playlistwin_popup_menu,
+							"/Queue - Unqueue");
+			gtk_widget_set_sensitive(w, sensitive);
 
-		playlistwin_set_sensitive_sortmenu();
-		playlistwin_set_sensitive_popupmenu();
-	
-		util_item_factory_popup_with_data(playlistwin_popup_menu,
-						  GINT_TO_POINTER(pos), NULL,
-						  event->x_root,
-						  event->y_root + 5,
-						  3, event->time);
+			playlistwin_set_sensitive_sortmenu();
+			playlistwin_set_sensitive_popupmenu();
+
+			util_item_factory_popup_with_data(playlistwin_popup_menu,
+							  GINT_TO_POINTER(pos),
+							  NULL, event->x_root,
+							  event->y_root + 5,
+							  3, event->time);
+		}
 		grab = FALSE;
 	}
 	else if (event->button == 4) /* Scrollwheel up */
@@ -1425,7 +1425,7 @@ static void playlistwin_physically_delete_cb(GtkWidget *widget, gpointer data)
 
 	if (message)
 	{
-		xmms_show_message(_("XMMS: Files deleted"), message, _("Ok"), FALSE, NULL, NULL);
+		xmms_show_message(_("XMMS: Files deleted"), message, _("OK"), FALSE, NULL, NULL);
 		g_free(message);
 	}
 }
@@ -1478,7 +1478,7 @@ static void playlistwin_physically_delete(void)
 	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), bbox, FALSE, FALSE, 0);
 
-	ok = gtk_button_new_with_label(_("Ok"));
+	ok = gtk_button_new_with_label(_("OK"));
 	cancel = gtk_button_new_with_label(_("Cancel"));
 	gtk_signal_connect(GTK_OBJECT(ok), "clicked", playlistwin_physically_delete_cb, selected_list);
 	gtk_signal_connect_object(GTK_OBJECT(ok), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dialog));
@@ -1501,7 +1501,8 @@ static void playlistwin_keypress_up_down_handler(PlayList_List *pl, gboolean up,
 	
 	if (pl->pl_prev_selected == -1 ||
 	    (!playlistwin_item_visible(pl->pl_prev_selected) &&
-	     !(state & GDK_SHIFT_MASK && pl->pl_prev_min != -1)))
+	     !(state & (GDK_SHIFT_MASK | GDK_MOD1_MASK) &&
+	       pl->pl_prev_min != -1)))
 	{
 		pl->pl_prev_selected = pl->pl_first;
 	}
@@ -1591,7 +1592,8 @@ static gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointe
 			playlistwin_list->pl_first = get_playlist_length() - playlistwin_list->pl_num_visible;
 			break;
 		case GDK_Return:
-			if (playlistwin_list->pl_prev_selected > -1 && playlistwin_item_visible(playlistwin_list->pl_prev_selected))
+		case GDK_KP_Enter:
+			if (playlistwin_list->pl_prev_selected > -1)
 			{
 				playlist_set_position(playlistwin_list->pl_prev_selected);
 				if (!get_input_playing())
@@ -1778,6 +1780,10 @@ static void playlistwin_sort_menu_callback(gpointer cb_data, guint action, GtkWi
 			playlist_random();
 			playlistwin_update_list();
 			break;
+		case PLAYLISTWIN_SORT_SEL_RANDOMIZE:
+			playlist_randomize_selected();
+			playlistwin_update_list();
+			break;
 	}
 }
 
@@ -1959,14 +1965,15 @@ void playlistwin_create(void)
 
 	playlistwin_accel = gtk_accel_group_new();	
 
-	playlistwin_sort_menu = gtk_item_factory_new(GTK_TYPE_MENU,
-						     "<Main>", NULL);
+	playlistwin_sort_menu = gtk_item_factory_new(GTK_TYPE_MENU, "<Main>",
+						     playlistwin_accel);
 	gtk_item_factory_set_translate_func(playlistwin_sort_menu,
 					    util_menu_translate, NULL, NULL);
 	gtk_item_factory_create_items(GTK_ITEM_FACTORY(playlistwin_sort_menu),
 				      playlistwin_sort_menu_entries_num,
 				      playlistwin_sort_menu_entries, NULL);
-	playlistwin_sub_menu = gtk_item_factory_new(GTK_TYPE_MENU, "<Main>", NULL);
+	playlistwin_sub_menu = gtk_item_factory_new(GTK_TYPE_MENU, "<Main>",
+						    playlistwin_accel);
 	gtk_item_factory_set_translate_func(playlistwin_sub_menu,
 					    util_menu_translate, NULL, NULL);
 	gtk_item_factory_create_items(GTK_ITEM_FACTORY(playlistwin_sub_menu),
@@ -2065,12 +2072,15 @@ static void playlistwin_popup_menu_callback(gpointer cb_data, guint action, GtkW
 				playlist_fileinfo(pos - 1);
 			break;
 		case MISC_QUEUE:
-			if (pos == 0)
-				/* We got here via an accelerator */
+			if (pos == 0 || (pos != -1 && playlist_is_position_selected(pos - 1)))
+				/* We got here via an accelerator or by right-clicking on a selection */
 				playlist_queue_selected();
 			else if (pos != -1)
 				playlist_queue_position(pos - 1);
 			break;
+		case MISC_QUEUE_MANAGER:
+			mainwin_queue_manager();
+			break; 
 		case SEL_LOOKUP:
 			playlist_read_info_selection();
 			break;
