@@ -400,7 +400,7 @@ void cleanup_plugins(void)
 	while (node)
 	{
 		InputPlugin *ip = node->data;
-		if (ip && ip->cleanup)
+		if (ip->cleanup)
 		{
 			ip->cleanup();
 			GDK_THREADS_LEAVE();
@@ -429,7 +429,7 @@ void cleanup_plugins(void)
 	while (node)
 	{
 		EffectPlugin *ep = node->data;
-		if (ep && ep->cleanup)
+		if (ep->cleanup)
 		{
 			ep->cleanup();
 			GDK_THREADS_LEAVE();
@@ -466,6 +466,7 @@ void cleanup_plugins(void)
 		node = node->next;
 	}
 	g_list_free(gp_data->general_list);
+	g_free(gp_data);
 
 	node = get_vis_enabled_list();
 	while (node)
@@ -491,5 +492,4 @@ void cleanup_plugins(void)
 	}
 	g_list_free(vp_data->vis_list);
 	g_free(vp_data);
-
 }
