@@ -307,7 +307,7 @@ void prefswin_rt_callback(GtkToggleButton * w, gpointer data)
 		  "security implications.\n\n"
 		  "Using this feature is not recommended.\n"
 		  "To activate this you need to restart XMMS."),
-		_("Ok"), FALSE, NULL, NULL);
+		_("OK"), FALSE, NULL, NULL);
 }
 
 static void prefswin_toggle_wm_decorations(void)
@@ -711,7 +711,7 @@ void create_prefs_window(void)
 	GtkWidget *options_font_hbox, *options_font_vbox;
 	GtkWidget *options_mouse_box, *options_mouse_label;
 	GtkObject *options_mouse_adj;
-	GtkWidget *prefswin_title_desc, *prefswin_title_label, *opt;
+	GtkWidget *prefswin_title_desc, *prefswin_title_label, *prefswin_moreinfo_label, *opt;
 
 	char *titles[1];
 
@@ -1009,6 +1009,10 @@ void create_prefs_window(void)
 	gtk_container_add(GTK_CONTAINER(options_pbs), options_pbs_box);
 
 	options_sw = prefswin_option_new_to_table(&cfg.snap_windows, GTK_TABLE(options_table), 1, 6);
+	gtk_tooltips_set_tip(prefswin_tooltips, options_sw,
+			     _("When moving windows around, snap them "
+			       "together, and towards screen edges at "
+			       "this distance"), NULL);
 	options_sw_box = gtk_hbox_new(FALSE, 5);
 	options_sw_label = gtk_label_new(_("Snap windows at"));
 	gtk_box_pack_start(GTK_BOX(options_sw_box), options_sw_label, FALSE, FALSE, 0);
@@ -1029,7 +1033,7 @@ void create_prefs_window(void)
 		GTK_TABLE(options_table), 1, 7);
 	gtk_tooltips_set_tip(prefswin_tooltips, opt,
 			     _("Recommended if you want to load playlists "
-			       "that was created in MS Windows"), NULL);
+			       "that were created in MS Windows"), NULL);
 
 	options_mouse_box = gtk_hbox_new(FALSE, 5);
 	options_mouse_label = gtk_label_new(_("Mouse Wheel adjusts Volume by (%)"));
@@ -1040,10 +1044,14 @@ void create_prefs_window(void)
 	gtk_box_pack_start(GTK_BOX(options_mouse_box), prefswin_options_mouse_spin, FALSE, FALSE, 0);
 	gtk_table_attach_defaults(GTK_TABLE(options_table), options_mouse_box, 0, 1, 8, 9);
 	
-	prefswin_option_new_with_label_to_table(&cfg.use_pl_metadata,
-						_("Use meta-data in playlists"),
-						GTK_TABLE(options_table), 1, 8);
+	opt = prefswin_option_new_with_label_to_table(&cfg.use_pl_metadata,
+						      _("Use meta-data in playlists"),
+						      GTK_TABLE(options_table), 1, 8);
+	gtk_tooltips_set_tip(prefswin_tooltips, opt,
+			     _("Store information such as song title and "
+			       "length to playlists"), NULL);
 
+	
 	gtk_notebook_append_page(GTK_NOTEBOOK(prefswin_notebook), prefswin_options_vbox, gtk_label_new(_("Options")));
 
 	/*
@@ -1117,11 +1125,29 @@ void create_prefs_window(void)
 	prefswin_title_desc = xmms_titlestring_descriptions("pagfFetndyc", 2);
 	gtk_box_pack_start(GTK_BOX(prefswin_title_vbox2), prefswin_title_desc, FALSE, FALSE, 0);
 
+
+	prefswin_title_frame = gtk_frame_new(_("Advanced Title Options"));
+	gtk_box_pack_start(GTK_BOX(prefswin_title_vbox), prefswin_title_frame, FALSE, FALSE, 0);
+	gtk_container_border_width(GTK_CONTAINER(prefswin_title_frame), 5);
+	prefswin_title_vbox2 = gtk_vbox_new(FALSE, 10);
+	gtk_container_add(GTK_CONTAINER(prefswin_title_frame), prefswin_title_vbox2);
+	gtk_container_border_width(GTK_CONTAINER(prefswin_title_vbox2), 5);
+
+	prefswin_moreinfo_label = gtk_label_new(_(
+	"%0.2n - Display a 0 padded 2 char long tracknumber\n"
+	"\n"
+	"For more details, please read the included README or "
+	"http://www.xmms.org/docs/readme.php"));
+
+	gtk_box_pack_start(GTK_BOX(prefswin_title_vbox2), prefswin_moreinfo_label, FALSE, FALSE, 0);
+	gtk_label_set_justify (GTK_LABEL(prefswin_moreinfo_label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment(GTK_MISC(prefswin_moreinfo_label), 0.0, 0.5);
+
 	gtk_notebook_append_page(GTK_NOTEBOOK(prefswin_notebook), prefswin_title_vbox, gtk_label_new(_("Title")));
 
 
 	/* 
-	 * Ok, Cancel & Apply 
+	 * OK, Cancel & Apply 
 	 */
 
 	prefswin_hbox = gtk_hbutton_box_new();
@@ -1129,7 +1155,7 @@ void create_prefs_window(void)
 	gtk_button_box_set_spacing(GTK_BUTTON_BOX(prefswin_hbox), 5);
 	gtk_box_pack_start(GTK_BOX(prefswin_vbox), prefswin_hbox, FALSE, FALSE, 0);
 
-	prefswin_ok = gtk_button_new_with_label(_("Ok"));
+	prefswin_ok = gtk_button_new_with_label(_("OK"));
 	gtk_signal_connect(GTK_OBJECT(prefswin_ok), "clicked", GTK_SIGNAL_FUNC(prefswin_ok_cb), NULL);
 	GTK_WIDGET_SET_FLAGS(prefswin_ok, GTK_CAN_DEFAULT);
 	gtk_box_pack_start(GTK_BOX(prefswin_hbox), prefswin_ok, TRUE, TRUE, 0);
